#ifndef PERMUTATION_H
#define PERMUTATION_H

#include "dsexceptions.h"
#include <iostream>        // For NULL
#include <vector>
#include <list>
#include <stack>
#include <assert.h>
#include <utility>
#include <limits.h>
#include "../tools/tools.h"
#include "../seqtools/seqfuncs.h"
#include "../seqtools/invdist.h"
//assume: 0 is never an element in the permutation
#define PHI 0

//The method to use for choosing the next oriented inversion:
#define USE_MIN 0
#define USE_MAX 1
#define RANDOM  3

// #define TRY_OTHER_ON_FAILURE  //If one oriented rev is unsafe, do the other.

#define NULL_IPAIR ipair(0,0)


using namespace std;

// Permutation class
//
// CONSTRUCTION: with no parameters
//
// ******************PUBLIC OPERATIONS*********************
// bool nextRev( )        --> Do the next sorting reversal (ret false otherwise)
// bool isEmpty( )        --> Return true if empty; else false
// void makeEmpty( )      --> Remove all items
// void printPerm( )      --> Print tree in sorted order
// void printNH( )        --> Print tree in New Hampshire format
// void printFlags( )     --> Print the flags on the nodes of the tree
// ******************ERRORS********************************
// Throws UnderflowException as warranted

typedef pair<int,int> ipair;
const ipair NULL_INDS = ipair(-1,1);

class Permutation
{
  friend ostream& operator<<(ostream& output, Permutation& p);

  public:
    Permutation(int len);              //Innitialize to the ID.
    Permutation(int* seq, int slen);
    Permutation(vector<int> seq);
    void initWith(int* seq, int slen);
    void initWith(vector<int> seq);
    //Permutation( const Permutation & rhs );
    ~Permutation( );


    // Sort the permutation and return the sorting sequence.
    list<ipair> getSortSeq(int flag = RANDOM);

    // Do the next sorting reversal (return false otherwise).
    // Return in retval the pair of indices over which the reversal was done.
    bool nextRev(int flag = RANDOM);
    bool nextRev(ipair &retval, int flag = RANDOM);

    // Reverse the permutation over the given indices.
    // p1...p(i-1) p(i)... p(j)p(j+1)...pn becomes
    // p1...p(i-1)-p(j)...-p(i)p(j+1)...pn
    void rev(int i, int j);
    void rev(ipair r);

    // Get a reversal that we know is safe, given the unsafe reversal -r- and
    // the interval of the created hurdle -interval-.
    ipair getSafeRev(ipair r, ipair interval) const;
    //ipair findMinMax(vector<int> &p, int i, int j) const;
    //bool getSafeRevLeft(ipair &safeRev,int hi,int ri,int rj,
                        //vector<int> &p, ipair &minmax) const;
    //bool getSafeRevRight(ipair &safeRev,int hj,int ri,int rj, vector<int> &p,
                         //ipair &minmax) const;
    ipair getSafeRevDumb(ipair r, ipair interval) const;
    // Return the index of the element in the given permutation if it is in the
    // given interval.  Otherwise, return 0.
    int getPositiveInd(vector<int> p, int elem, ipair interval) const;

    /**
     * Find the smallest item in the tree.
     * Not the most efficient implementation (uses two passes), but has correct
     *     amortized behavior.
     * A good alternative is to first call find with parameter
     *     smaller than any item in the tree, then call findMin.
     * Return the smallest item or throw UnderflowException if empty.
     */
    int findMin( );

    /**
     * Find the largest item in the tree.
     * Not the most efficient implementation (uses two passes), but has correct
     *     amortized behavior.
     * A good alternative is to first call find with parameter
     *     larger than any item in the tree, then call findMax.
     * Return the largest item or throw UnderflowException if empty.
     */
    int findMax( );

    bool contains( int index );
    bool isEmpty( ) const;
    vector<int> getSeqVec( ) const;
    // Get the sequence vector with frame elements.
    vector<int> getFramedSeqVec() const;
    void printPerm( ) const;
    void printNH() const;
    void printFlags( ) const;
    void printCompInfo( ) const;
    // Print the paths from every node to the root.
    void printNodesToRoot() const;

    // Return true if the permutation is the identity.
    bool isSorted();

    void makeEmpty( );
    //void makeEmptySlow( );

    //void insert( int index, int x);
    void remove( int index );

    inline int length();
    //const Permutation & operator=( const Permutation & rhs );

    //Input: Permutation of n-1 elements framed by 0 and n
    //Output: Indices of endpoints of each unoriented component in the
    //        permutation or empty vector if there are none.
    //vector<ipair> getUnorientedComps() const;

    //Return true if there is a hurdle.
    bool hasHurdle() const;

    // Apply the given reversals to the permutation in reverse order.
    void applyRevsLastTOFirst(list<ipair> revs);


private:
    struct BinaryNode
    {
        BinaryNode *left, *right, *parent;
        int element;
        int minNegative, maxNegative; //min max negative element in subtree
        int minPositive, maxPositive; //min max positive element in subtree
        int index, size;              //index in permutation, size of subtree
        bool reversed;                //true if the subtree is reversed
        bool stale;                   //true if the indices below are stale
             //Used only if resolving unsafe inversions:
             //(the nodes of the tree represent reality edges)
        int clab;                     //the comp label (phantom or otherwise)
        bool bp;                      //True if we're a breakpoint.

        
        BinaryNode() : left( NULL ), right( NULL ), parent( NULL ),
                       element( 0 ), minNegative(INT_MAX), maxNegative(PHI),
                       minPositive(INT_MAX), maxPositive(PHI), index(-1),
                       size(0), reversed(false), stale(false), clab(-1),
                       bp(false) { }
        
        BinaryNode( int ind, BinaryNode *lt, BinaryNode *rt, BinaryNode *pt,
                    int theElement, bool rev, bool s)
            : left( lt ), right( rt ), parent( pt ), element( theElement ), 
              index(ind), size(1), reversed(rev), stale(s), clab(-1),
              bp(false)
        {
             if (theElement < 0){
                 minNegative = maxNegative = 0 -theElement;
                 maxPositive = maxPositive = PHI;
             }
             else{
                 minNegative = maxNegative = PHI;
                 maxPositive = minPositive = theElement;
             }        
        }
        
        BinaryNode( int ind, int theElement )  //NOT USED!!!!!!!!
            : element( theElement ), index(ind), size(1), reversed(false),
              stale(false), clab(-1), bp(false)
        {
            if (theElement < 0)
            {
                minNegative = maxNegative = 0 - theElement;
                maxPositive = minPositive = PHI;
            }
            else
            {
                minNegative = maxNegative = PHI;
                maxPositive = minPositive = theElement;
            }        
        }
        
    };


    BinaryNode *root;            //The root of the tree.
    BinaryNode *nullNode;        //The node that every null link goes to.

    vector<BinaryNode *> lookup; //lookup[i] points to node element abs(i).



    BinaryNode *mem;             //The memory for the entire tree.
    //Insert a node with given index using memory from -mem- at postion -j-.
    void insert( int index, int x, int &j);

    /**
     * Internal method to reclaim internal nodes in subtree t.
     * WARNING: This is prone to running out of stack space.
     */
    void reclaimMemory( BinaryNode *t );
    
    /**
     * Internal method to print a subtree t in new hampshire format.
     * WARNING: This is prone to running out of stack space.
     */
    void printNH(BinaryNode *t, bool rev = false) const;

    /**
     * Internal method to print a subtree t in sorted order.
     * WARNING: This is prone to running out of stack space.
     */
    void printFlags( BinaryNode *t ) const;

    // Print the values of the component related flags.
    void printCompInfo(BinaryNode *t, bool rev=false) const;
 
    /**
     * Internal method to print a subtree t in sorted order.
     * WARNING: This is prone to running out of stack space.
     */
    void printPerm( BinaryNode *t ) const;

    vector<int> getSeqVec(BinaryNode *t) const;
    void getSeqVecRec(BinaryNode *t,vector<int> &ret,bool rev=false) const;
    // Get a vector of pointers (to the nodes) in permutation order.
    vector<BinaryNode*> getPermPointers() const;
    vector<BinaryNode*> getPermPointers(BinaryNode *t) const;
    void getPermPointersRec(BinaryNode *t, vector<BinaryNode*> &ret,
                         bool rev = false) const;

    // Print the permutation ignoring the changes from the reversed flags.
    void printWrongPerm( BinaryNode *t) const;

    // Back out from a sequence of inversions until the most recent unsafe
    // inversion is met.
    // Return the reversals that must be undone from the identity.
    // The front-most is the unsafe inversion that was done.
    list<ipair> getUnstuck(Permutation &op, list<ipair> &revs);

    // Find the HP graph and all oriented components into adjacencies.
    // Return the corresponding permutation.
    void removeOrientedComps(vector<int> &p) const;

    // Return the elements of the adjacency created for -r- on this permutation.
    ipair getAdjCreated(ipair r);

    // Return the corresponding reversal on the inverse permutation.
    ipair findInvRev(ipair r);

    // Return true if p is nested in q.
    inline bool isnested(ipair p, ipair q);
    inline bool intersects(ipair p, ipair q);

    /**
     * Internal method to clone subtree.
     * WARNING: This is prone to running out of stack space.
     */
    //BinaryNode * clone( BinaryNode *t ) const;

    BinaryNode * findMinP( );
    BinaryNode * findMaxP( );

        // Tree manipulations
    void rotateWithLeftChild( BinaryNode * & k2 );
    void rotateWithRightChild( BinaryNode * & k1 );

    /**
     * Internal method to perform a top-down splay.
     * The last accessed node becomes the new root.
     * This method may be overridden to use a different
     * splaying algorithm, however, the splay tree code
     * depends on the accessed item going to the root.
     * x is the target index to splay around.
     * t is the root of the subtree to splay.
     */
    void splay( int index, BinaryNode * & t );

    // Swap the subtrees and flip their flags.
    inline void pushRevDown(BinaryNode *t);
    // Fix the index of this node based on it's parent's index and the size of
    // the subtrees below it.
    inline void pushIndDown(BinaryNode *t);
    // Swap the min and max holder for the node.
    void swapMM(BinaryNode *t);

    // Assure that the given node has its index and sign updated.
    void updateNodeInfo(BinaryNode* j);

    inline bool isAllPositive();
    //go down the tree in the direction of minNeg or maxNeg, modifies i
    //break if we reach the node with minNeg/maxNeg value
    inline bool stepDown(BinaryNode* &i, int flag);

    inline bool isEndElement(BinaryNode* i, ipair &retval, int flag);
    inline BinaryNode* getOtherEnd(int elem, int flag);
    inline void fixIndices(ipair &retval, int i, int j, int flag);

    //find index of min element (ignoring sign) in the interval [i,j]
    inline int findMin(vector<int> &p, int i, int j);

    // Update the given enterval indices if it is a subinterval of the -rev-.
    void updateInterval(const ipair rev, ipair &interval) const;

    // Return true if the reversal given would result in the same number of
    // unoriented components.
    // ** this permutation must represent the reality edges and have clab
    //    and bp labelled properly!
    bool isSafe(const ipair i1);

    // Get a component that would be broken by the given reversal.
    // ** assume the only components are the unoriented ones.
    ipair getBrokenComp(ipair r);

    // Set the clab for the given node.
    //void setCnum(int val, BinaryNode* node);

    // Return the endpoints given by the method in -flag-.
    ipair doOrientedRev(int flag);

    // Return the inversion associated with the given endpoint.
    ipair getOtherEndpoint(BinaryNode* i, int flag);


    inline int MAX_N (int root, int left, int right);
    inline int MAX_P (int root, int left, int right);
    inline int MIN_N (int root, int left, int right);
    inline int MIN_P (int root, int left, int right);

    inline void updateFields(BinaryNode *t);
    void updateFieldsInPath(BinaryNode *t);

        //Split the tree (rootel at -root-) at the given index.
        //Do this by splaying the index to the root and then chopping
        //the right child of the root.
    BinaryNode * split(int index, BinaryNode *&root);

        //assume: all indices of root < all indices of root2
        //root modified, root2 is made right subtree of root
        //changed root returned
    BinaryNode * join(BinaryNode *&root, BinaryNode *root2);

        //Undo reversals until the most recent unsafe.
    void fixUnsafe(int &k, vector<int> &original, Permutation &op,
                   list<ipair> &beginning, list<ipair> &ending);

        //Finish filling out the reversal indicies from the list
        //of oriented pairs.
    void finalizeSortSeq(list<ipair> &sortseq, Permutation &p,
                         list<ipair> &beginning, list<ipair> &ending);

};

#endif
