/* Krister Swenson, Joel V. Earnest-DeYoung                  January 2004
A CompInfo contains a list of cycle labels.
You must tell it which nodes are part of what cycle using markCycle().
  `-> markCycle() must be called for every value of node before the CompInfo
      can be built.
*/

#include "CompInfo.h"
// #define VERBOSE

using namespace std; 
using namespace __gnu_cxx; 


//----------------- Contructors etc. ---------------------------

const Hurdle CompInfo::nullHurdle(true, -1, -1, -1);

CompInfo::CompInfo(int nodes)
  {
  numcycles = 0;
  numcomps = 0;
  numnodes  = nodes;
  ready = false;
  staleComp = true;
                              //We don't fill this in until classCycles().
  components = (Component **)malloc((numnodes/2)*sizeof(Component*));

  cyccomp = new int[nodes];
  cyclenums = new int[nodes]; //Initialize cycle and component number arrays:
  compnums = new int[nodes];
  for(int i=0; i < nodes; i++)
    {
    cyclenums[i] = -1;
    compnums[i] = -1;
    cyccomp[i] = -1;
    }

  simHurdles.clear();
  superhurdles.clear();
  syncHurdles();
  maximal = false;
  maxLeftMax = -1;
  maxRightMin = -1;
  fortress = false;
  neoLeftMax = -1;
  }

CompInfo::CompInfo(const CompInfo& c)
  { copy(c); }

CompInfo::~CompInfo()
  { del(); }

void CompInfo::operator=(const CompInfo& c)
  { del(); copy(c); }

void CompInfo::del()
  {
  int i;

  delete [] cyclenums;
  delete [] compnums;
  delete [] cyccomp;
     
  for(i=0; i < numcomps; i++)
    delete components[i];
  free(components);
  }

void CompInfo::copy(const CompInfo& c)
  {
  int i;

  ready = c.ready;
  staleComp = c.staleComp;
  numcycles = c.numcycles;    //The number of cycles to be put into components.
  numnodes = c.numnodes;      //The number of nodes in the graph.
  numcomps = c.numcomps;      //The number of components in the graph.
                              //The cycle number each node has:
  cyclenums = new int[numnodes];
  compnums = new int[numnodes];
  for(i=0; i < numnodes; i++)
    {
    cyclenums[i] = c.cyclenums[i];
    compnums[i] = c.compnums[i];
    }

  cyccomp = new int[numcycles];
  for(i=0; i < numcycles; i++)
    cyccomp[i] = c.cyccomp[i];

  cycles = c.cycles;

  components = (Component **)malloc((numnodes/2)*sizeof(Component*));
  for(i=0; i < numcomps; i++)
    {
    components[i] = new Component(1);
    *components[i] = *c.components[i];
    }

  simHurdles = c.simHurdles;
  superhurdles = c.superhurdles;
  syncHurdles();	      // Set hurdles using simHurdles and superhurdles

  maximal = c.maximal;
  maxLeftMax = c.maxLeftMax;
  maxRightMin = c.maxRightMin;
  fortress = c.fortress;
  neoLeftMax = c.neoLeftMax;
  }

//--------------------------------------------------------------



/* Mark that the cycle containing this node has a deletion.
   ** Temporary.
*/
void CompInfo::setCycleDelVal(int node)
  { cycles[getCycleNum(node)].setDel(); }


/* Return true if the given cycle is marked with deletions.
   ** Temporary.
*/
bool CompInfo::getCycleDelVal(int cycle)
  { return cycles[cycle].getDel(); }



/* Return the number of the cycle that -node- is a part of.
*/
int CompInfo::getCycleNum(int node) const
  { return cyclenums[node]; }

/* Return the size of the cycle that -node- is a part of.
*/
int CompInfo::getCycleSize(int node)
  { return cycles[cyclenums[node]].getSize(); }

/* Return the size of the largest cycle.
*/
int CompInfo::getLargestCycleSize()
  {
  int largest=0;

  for(cyclemap::iterator c = cycles.begin(); c != cycles.end(); c++)
    {
    int clen = c->second.getSize()/2;
    if(clen > largest)
      largest = clen;
    }

  return largest;
  }

/* Return a vector of size of the cycles, sorted from largest to smallest.
*/
vector<int> CompInfo::getCycleSizes()
  {
  vector<int> retval;
  for(cyclemap::iterator c = cycles.begin(); c != cycles.end(); c++)
    retval.push_back(c->second.getSize()/2);
  sort(retval.begin(), retval.end());
  reverse(retval.begin(), retval.end());

  return retval;
  }

/*Return a random node on the given cycle.
   ** Do srandom((unsigned int)time(NULL)) first.
*/
int CompInfo::getRandomNodeOnCycle(int num)
  {
  int n = random() % cycles[num].getSize();
  return cycles[num].getNodeNum(n);
  }


/*Get a transposition that makes a cycle split into two, otherwise, return 0.
*/
int CompInfo::getTwoSplitTrans(ipair &interval)
  {
  for(cyclemap::iterator i=cycles.begin(); i!=cycles.end(); i++)
    {
    Cycle *c = &(i->second);
    if(c->getSize() > 4)              //If the cycle is long enough
      {                               //try each left endpoint:
      for(int l=0; l < c->getSize(); l+=2)
        for(int r=l+4; r < l+c->getSize(); r+=4)
          {                           //For each right endpoint an even number
          int right = r%c->getSize(); //or steps away:
          if(c->getNodeNum(right) > c->getNodeNum(l))
            {                         //Get a possible interval and
                                      //find a place to put it:
                                      //Try to place one desire edge away:
            int b = c->getNodeNum((l+2)%c->getSize());
            if((b > c->getNodeNum(right)) || (b < c->getNodeNum(l)))
              {                       //If it's not in the interval
              interval.first = (c->getNodeNum(l)/2)+1;
              interval.second = c->getNodeNum(right)/2;
              return (b/2)+1;         //then return this configuration.
              }
            }
          }
      }
    }
  return 0;
  }





/* Return the number of the component that -node- is a part of.
*/
int CompInfo::getCompNum(int node)
  { return compnums[node]; }


/*Get the number of bad (unoriented) components.
*/
int CompInfo::getBadCompCount()
  {
  if(staleComp)
    classCycles();

  int count=0;
  for(int i=0; i<numcomps; i++)
    if(!components[i]->isGood())
      count++;
  return count;
  }


/*Mark -node- as being part of cycle number -cyc- and having the
value -permval- in the permutation.
 ** we assume that if -cyc- is higher than any cycle that currently
    exists, then it it one higher than the highest.
*/
void CompInfo::addToCycle(int node, int permval, int cyc)
  {
  if(cyc >= numcycles)                 //If there isn't a cycle of this number, 
    {
    cycles[cyc] = Cycle();             //make one
    numcycles++;                       //and increment the number of cycles.
    }
    
  cyclenums[node] = cyc;               //Mark the cycle number for this node.
  cycles[cyc].add(node, permval);      //Add this node to the cycle.
  }

/*Merge the two cycles -c1- and -c2-.
*/
void CompInfo::shrinkCycles(int c1, int c2)
  {
  int j=0;                //Copy info for c2 to c1.
  for(vector<int>::const_iterator i = cycles[c2].getBeg();
                                  i != cycles[c2].getEnd(); i++)
    {
    cyclenums[*i] = c1;
    cycles[c1].add(*i, cycles[c2].getPermNum(j++));
    }
  cycles[c2].clear();     //Erase the c2 cycle.
  ready = false;          //Invalidate this object.
  }


/*Return the subset of oriented inversions from -set- that are on component -c-.
*/
list<Pair> CompInfo::invsONcomp(int c, const list<Pair> &set) const
  {
  assert(!staleComp);     //Make sure the info is not out of date.
  list<Pair> retlist;

  for(list<Pair>::const_iterator i = set.begin(); i != set.end(); i++)
    if(compnums[(i->getLeft()*2)-1] == c) //If the left index operates upon
      retlist.push_back(*i);              //component -c- then it's a keeper.

  return retlist;
  }

/*Return the subset of oriented inversions from -set- that are on
  the same component as all inversions in -invs-.
*/
list<Pair> CompInfo::invsONcomp(const list<Pair> &invs,
                                const list<Pair> &set) const
  {
  hash_set<int> comps;
  for(list<Pair>::const_iterator i = invs.begin(); i != invs.end(); i++)
    comps.insert(compnums[(i->getLeft()*2)-1]);

  list<Pair> retval;
  for(list<Pair>::const_iterator i = set.begin(); i != set.end(); i++)
    if(comps.find(compnums[(i->getLeft()*2)-1]) != comps.end())
      retval.push_back(*i);

  return retval;
  }



/*Make the given cycle good.
*/
void CompInfo::setGood(int cycnum)
  { cycles[cycnum].setGood(true); }

void CompInfo::printCycle(int cyc)
  { cerr << "\nCycles: " << cycles[cyc] << endl << endl; }

void CompInfo::printCycles()
  {
  for(int i=0; i<numcycles; i++)
    printCycle(i);
  }

/*Return the number of cycles over all components.
*/
int CompInfo::getNumCycles()
  { return numcycles; }

/*Return the number of components.
*/
int CompInfo::getNumComps()
  { return numcomps; }





/*Put the cycles into their respective components and mark the components good
(oriented) if one of it's constituent cycles is good.
--Efficiency -> set min and max in markCycle().
*/
void CompInfo::classCycles()
  {
  int i;
  CycleInteractions *cints;
  cints = (CycleInteractions *)malloc(numcycles*sizeof(CycleInteractions));
  for(i=0; i < numcycles; i++)            //Initialize -cints-:
    {
    cints[i].comp = -1;
    cints[i].min = numnodes;
    cints[i].max = -1;
    cints[i].tangled = new bool[numcycles];
    cints[i].within = new bool[numcycles];
    for(int j=0; j < numcycles; j++)
      {
      cints[i].within[j] = false;
      cints[i].tangled[j] = false;
      }
    }

  for(i=0; i < numnodes; i++)		  //Find -min- and -max-:
    {
    if(cints[cyclenums[i]].min > i)
      cints[cyclenums[i]].min = i;
    if(cints[cyclenums[i]].max < i)
      cints[cyclenums[i]].max = i;
    }
  for(i=0; i < numcycles; i++)		  //Find the cycles that touch within:
    for(int j=cints[i].min+2; j < cints[i].max-1; j=j+2)
      cints[i].within[cyclenums[j]] = true;

  for(i=0; i < numcycles; i++)		  //Find the cycles that are entangled:
    {
    for(int j=0; j < cints[i].min; j=j+2) //Check before [min...max]:
      if(cints[i].within[cyclenums[j]] == true)   //If the cycle at this node
        {                                         //occurred within cycle -i-,
        cints[i].tangled[cyclenums[j]] = true;    //then mark each as being
        cints[cyclenums[j]].tangled[i] = true;    //entangled with the other.
        }

    for(int j=cints[i].max+1; j < numnodes; j=j+2)
      if(cints[i].within[cyclenums[j]] == true)   //If the cycle at this node
        {					  //occurred within cycle -i-,
        cints[i].tangled[cyclenums[j]] = true;    //then mark each as being
        cints[cyclenums[j]].tangled[i] = true;    //entangled with the other.
        }
    }

//------------OUTPUT------------------------
// for(i=0; i < numcycles; i++)
//   {
//   cerr << endl;
//   cerr << i << ": " << cints[i].min << "-" << cints[i].max << "  WITHIN:";
//   for(int j=0; j < numcycles; j++)
//     cerr << cints[i].within[j] << " ";
//   cerr << "    TANGLED: ";
//   for(int j=0; j < numcycles; j++)
//     cerr << cints[i].tangled[j] << " ";
//   }
//------------------------------------------

  delete [] cyccomp;
  cyccomp = new int[numcycles];    //Get cyccomp ready to be filled.
  int compnum = 0;		   //The current component we are building.
  for(i=0; i < numcycles; i++)
    if(cints[i].comp < 0)	   //If this cycle isn't part of a component yet
      {				   // build a component off this cycle.
      buildComp(compnum, i, cints);
      compnum++;
      }

  for(i=0; i < numnodes; i++)      //Fill compnums:
    compnums[i] = cyccomp[cyclenums[i]];

  for(i=0; i < numcycles; i++)     //Take care of the CycleInteractions memory:
    {
    delete [] cints[i].within;
    delete [] cints[i].tangled;
    }
  free(cints);

  staleComp = false;
  }

/*Find the components that are hurdles and superhurdles. Figure out if graph
 *is a fortress. Return enough information to eliminate all of these 
 *structures.
 */
void CompInfo::findHurdles()
{
  // Precondition: 
  //  The gene sequence must be a positive permutation, or equivalently all
  //  components with more than one reality edge must be unoriented.
  //  ** For example, check that allParityZero()==true before calling **

  if(staleComp)    //Not sure if joel actually uses the -components- structure.
    classCycles();

  int numreality = numnodes/2;		// Number of reality edges
  int curr;				// Component of current reality edge
  int last = 0;				// Last unoriented component
  stack<int> s;
  int top;				// Top of the stack
  int minComp = -1;			// Minimum unoriented component

  vector<int> min(numcomps);		// Minimum seq index of the component
  vector<int> max(numcomps);		// Maximum seq index of the component

  vector<bool> isUnoriented(numcomps);	// Note: All components with multiple
					//  reality edges are unoriented
  Hurdle h;

  vector<int> numSpanned(numcomps);	// Number of unoriented components a
					//  component spans
  vector<int> spans(numcomps);		// Index of a spanned component
					//  (unique if numSpanned==1)
  int spanned;				// Index of spanned component
  vector<bool> separates(numcomps);	// True if component separates hurdles
					// (Hannenhalli-Pevzner meaning of
					//  "separates")
  bool mightSeparate;

  int neo = -1;				// "Neo-hurdle" component number
  int i;

  h.isNull = false;
  maximal = false;
  maxLeftMax = -1;
  maxRightMin = -1;
  fortress = false;
  neoLeftMax = -1;

  for (i=0; i < numcomps; ++i) {
    isUnoriented[i] = false;
    simHurdles.clear();
    superhurdles.clear();
    numSpanned[i] = 0;
    spans[i] = -1;
    separates[i] = false;
  }
  
  // Identify the locations and types of hurdles (single pass over compnums)
  for (i=0; i < numreality; ++i) {
    curr = compnums[2*i];
    if (s.empty() || (top=s.top()) < curr) { // new component
      s.push(curr);
      min[curr] = i;
      max[curr] = i+1;
    }
    else {				// repeat component
      max[curr] = i+1;
      isUnoriented[curr] = true;
      last = curr;
      if (minComp == -1 || curr < minComp)
	minComp = curr;

      if (curr < top) {			// old component
	if (numSpanned[curr] > 0)
	  mightSeparate = true;
	else
	  mightSeparate = false;

	// clear all spanned components from the stack
	while ((top=s.top()) > curr) {
	  s.pop();  
	  if (isUnoriented[top]) {
	    if (numSpanned[top]==0) {	// top is a hurdle
	      h.setVars(top, min[top], max[top]);
	      simHurdles.insert(simHurdles.end(), h);
	    }
	    
	    if (numSpanned[top]==1 && !simHurdles.empty()) {
	      spanned = spans[top];
	      if (spans[spanned]==-1) {	// spanned is a superhurdle
		h.setVars(spanned, min[spanned], max[spanned]);
		superhurdles.insert(superhurdles.end(), h);
		simHurdles.erase(h);
	      }
	    }

	    // update current component's spanning and separating info
	    spans[curr] = top;
	    ++numSpanned[curr];
	    if (mightSeparate)
	      separates[curr] = true;
	  }
	}
      }
    }
  }

  // Close out the last items on the stack
  while (!s.empty()) {
    top = s.top();
    s.pop();
    if (isUnoriented[top]) {
      if (numSpanned[top]==0) {		// top is a hurdle
	h.setVars(top, min[top], max[top]);
	simHurdles.insert(h);
      }
		    
      if (numSpanned[top]==1 && !simHurdles.empty()) {
	spanned = spans[top];
	if (spans[spanned]==-1) {	// spanned is a superhurdle
	  h.setVars(spanned, min[spanned], max[spanned]);
	  superhurdles.insert(h);
	  simHurdles.erase(h);
	}
      }
    }
  }

  // Finally, check for a maximal hurdle
  h.setVars(last, min[last], max[last]);
  if (last==minComp && !separates[last] 
      && simHurdles.find(h)==simHurdles.end()) {
    maximal = true;

    // insert the maximal hurdle into the appropriate set
    if (numSpanned[last]==1 && !separates[spans[last]])
      superhurdles.insert(h);
    else
      simHurdles.insert(h);

    // find the max index of the left side of the maximal hurdle
    for (i = min[last]; i < numreality; ++i) {
      curr = compnums[2*i];

      if (curr==last)
        maxLeftMax = i+1;
      else if (isUnoriented[curr])
        break;
    }

    // find the min index of the right side of the maximal hurdle
    for (i = max[last]-1; i > 0; --i) {
      curr = compnums[2*i];

      if (curr==last)
        maxRightMin = i;
      else if (isUnoriented[curr])
        break;
    }
  }

  // If the genome is a fortress
  if (simHurdles.empty() && superhurdles.size() > 2 
      && superhurdles.size() % 2 == 1) {
    fortress = true;

    // find the component number of a "neo-hurdle"
    // (This is the hurdle formed in clearing the fortress. In this case I'm
    //  using the outermost component which concentrically spans the 
    //  2nd-leftmost superhurdle)
    vector<int> spannedBy(numcomps);
    for (i=0; i < numcomps; ++i)
      spannedBy[i] = -1;

    for (i=0; i<numcomps; ++i)
      if (numSpanned[i]==1)
	spannedBy[spans[i]] = i;	// only spans[i] is spanned by i

    neo = spannedBy[(++superhurdles.begin())->comp];
    while ((curr=spannedBy[neo]) != -1) {
      neo = curr;
    }

    // find the max index of the left side of a "neo-hurdle"
    for (i = min[neo]; i < numreality; ++i) {
      curr = compnums[2*i];

      if (curr==neo)
	neoLeftMax = i+1;
      else if (isUnoriented[curr])
	break;
    }
  }

  syncHurdles();

  // Minor Note: In several cases of 2 superhurdles, simple hurdles will be 
  //             classified as superhurdles and vice versa.
  //             Since the 2 hurdles will be just be merged either way, I don't
  //             think this needs fixing.

//------------OUTPUT------------------------
//   cerr << "compnums:";
//   for (int i=0; i < numnodes/2; ++i)
//     cerr << compnums[i*2] << " ";
//   cerr << endl;

//   cerr << "min:";
//   for (int i=0; i < numcomps; ++i)
//     cerr << min[i] << " ";
//   cerr << endl << "max:";
//   for (int i=0; i < numcomps; ++i)
//     cerr << max[i] << " ";
//   cerr << endl;

//   cerr << "isUnoriented:";
//   for (int i=0; i < numcomps; ++i)
//     cerr << isUnoriented[i] << " ";
//   cerr << endl;

//   cerr << "numSpanned:";
//   for (int i=0; i < numcomps; ++i)
//     cerr << numSpanned[i] << " ";
//   cerr << endl;

//   cerr << "spans:";
//   for (int i=0; i < numcomps; ++i)
//     cerr << spans[i] << " ";
//   cerr << endl;

//   cerr << "separates:";
//   for (int i=0; i < numcomps; ++i)
//     cerr << separates[i] << " ";
//   cerr << endl;

#ifdef VERBOSE
  if (simHurdles.size() > 0) {
    cerr << "Hurdles:";
    for (set<Hurdle>::iterator i=simHurdles.begin(); i != simHurdles.end(); 
	 ++i)
      cerr << "(" << i->comp << "=" << i->min << "," << i->max << ") ";
    cerr << endl;
  }
  if (superhurdles.size() > 0) {
    cerr << "Superhurdles:";
    for (set<Hurdle>::iterator i=superhurdles.begin();
	 i != superhurdles.end(); ++i)
      cerr << "(" << i->comp << "=" << i->min << "," << i->max << ") ";
    cerr << endl;
  }
  if (maximal)
    cerr << "Maximal hurdle with inner indices = (" << maxLeftMax << ","
	 << maxRightMin << ")" << endl;
  if (fortress)
    cerr << "Fortress. Using component " << neo 
	 << " as neo-hurdle with inner left index = " << neoLeftMax << endl;
#endif //VERBOSE

#ifdef SORTING_SEQUENCE
  if (globalSortSequence) {
    if (simHurdles.size() > 0)
      for (set<Hurdle>::iterator i=simHurdles.begin(); i != simHurdles.end();
	   ++i)
	globalSortSequence->addHurdle(i->min, i->max, false);
    if (superhurdles.size() > 0)
      for (set<Hurdle>::iterator i=superhurdles.begin();
	   i != superhurdles.end(); ++i)
	globalSortSequence->addHurdle(i->min, i->max, true);
    if (maximal)
      globalSortSequence->setMaximalHurdle(maxLeftMax, maxRightMin);
    if (fortress)
      globalSortSequence->setFortressRevPoint(neoLeftMax);
  }
#endif //SORTING_SEQUENCE
//------------------------------------------
}


/*Mark the component info as stale.
*/
void CompInfo::markStale()
  { staleComp = true; }

/*Return true if the component info is stale.
*/
bool CompInfo::isStale() const
  { return staleComp; }


/* Build the component -compid- starting with cycle -cycleid- using the info
in -cinfo-.  Do it depth first.

--Efficiency: get rid of recursion.
*********Check out the passing of cinfo.
*/
void CompInfo::buildComp(int compid, int cycleid, CycleInteractions cinfo[])
  {
  cinfo[cycleid].comp = compid;
  addToComp(cycleid, compid);
  for(int j=0; j < numcycles; j++)
    if(cinfo[cycleid].tangled[j] && (cinfo[j].comp < 0))
      buildComp(compid, j, cinfo);
  }


/*Add the given -cycle- to the component -comp-.
*/
void CompInfo::addToComp(int cycle, int comp)
  {
  if(comp < numcomps)
    {
    components[comp]->makeMember(cycle);
    cyccomp[cycle] = comp;
    }
  else
    {
    components[comp] = new Component(numcycles);
    components[comp]->makeMember(cycle);
    cyccomp[cycle] = comp;
    numcomps++;
    }
  if(cycles[cycle].getGood())
    components[comp]->makeGood();
  }


/*Return true if -ready- has been set or -cyclenums- is fully initialized.
*/
bool CompInfo::checkIfReady()
  {
  return ready || checkNodecycles();
  }

/*Return true and set -ready- if -cyclenums- is fully initialized.
*/
bool CompInfo::checkNodecycles()
  {
  int highest = 0;
  for(int i=0; i < numnodes; i++)
    {
    if(cyclenums[i] < 0)
      return false;

    if(cyclenums[i] > highest)
      highest = cyclenums[i];
    }
  ready = true;
  return true;
  }

ostream& operator<<(ostream& output, CompInfo& c)
  {
  if(c.checkIfReady())
    {
    int i;

    output << "COMPONENTS:  " << endl;
    output << "        0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7"
           << " 8 9 0 1 2 3\n";
    output << "Cycles: ";
    for(i=0; i < c.numnodes; i++)
      output << c.cyclenums[i] << " ";

    c.classCycles();

    output << endl << "Cycle/Component nums: ";
    for(i=0; i < c.numcycles; i++)
      output << c.cyccomp[i] << " ";
    output << endl;
    output << "            0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5"
           << " 6 7 8 9 0 1 2 3\n";
    output << "Components: ";
    for(i=0; i < c.numnodes; i++)
      output << c.compnums[i] << " ";
    }
  else
    output << "Error[CompInfo::<<]: -cyclenums- not full yet!" << endl;

  return output;
  }

/*Set hurdles as the union of simHurdles and superhurdles
 */
void CompInfo::syncHurdles() {
  hurdles.empty();
  hurdles = simHurdles;
  hurdles.insert(superhurdles.begin(),superhurdles.end());
  currHurdle = hurdles.begin();
  currSimH = simHurdles.begin();
  currSupH = superhurdles.begin();
}

// Return the first hurdle (simple of super)
Hurdle CompInfo::get1stHurdle() {
  currHurdle = hurdles.begin();
  if (currHurdle == hurdles.end())
    return nullHurdle;
  else
    return *currHurdle;
}

//Return the next hurdle (simple or super)
Hurdle CompInfo::getNextHurdle() {
  if (currHurdle==hurdles.end() || ++currHurdle==hurdles.end())
    return nullHurdle;
  else
    return *currHurdle;
}

//Return the leftmost simple hurdle
Hurdle CompInfo::get1stSimHur() {
  currSimH = simHurdles.begin();
  if (currSimH == simHurdles.end())
    return nullHurdle;
  else
    return *currSimH;
}

//Return the next simple hurdle
Hurdle CompInfo::getNextSimHur() {
  if (currSimH==simHurdles.end() || ++currSimH==simHurdles.end())
    return nullHurdle;
  else
    return *currSimH;
}

//Return the leftmost superhurdle
Hurdle CompInfo::get1stSupHur() {
  currSupH = superhurdles.begin();
  if (currSupH == superhurdles.end())
    return nullHurdle;
  else
    return *currSupH;
}

//Return the next superhurdle
Hurdle CompInfo::getNextSupHur() {
  if (currSupH==superhurdles.end() || ++currSupH==superhurdles.end())
    return nullHurdle;
  else
    return *currSupH;
}
