#ifndef COMPINFO_H
#define COMPINFO_H

/* Krister Swenson                                           Spring 2003
A CompInfo contains a list of Components and Cycles.  The components refer
the their constituent cycles by cycle numbers (indices into the Cycle list).
You must tell it which nodes are part of what cycle using markCycle().
  `-> markCycle() must be called for every value of node before the CompInfo
      can be built.

It first gets a list of cycle identities for each node of the graph in
-cyclenums- through markCycle().  classCycles() uses this list and marks
two cycles as being entangled(part of the same component) iff each of the 
cycles shows up in cyclenums on both sides of the other (ie their desire edges
cross).

Example from Setubal+Medianis:
       sequence:   L 3 -3 -4 4 1 -1 2 -2 -5 5 R
       cyclenums = 0 0  0  0 1 1  0 0  1  1 2 2
       `-> cycle 0 shows up on either side of cycle 1 and vice versa,
           so cycle 0 and 1 are entangled.

The components finally, are built from the entanglements.
*/

#ifdef _MSC_VER
#pragma warning (disable : 4503) // Turn off dumb Visual C++ warning
#endif

#include <iostream>
#include <cstdlib>
#include <cstdio>
#include <assert.h>
#include <stack>
#include <set>
#include <vector>
#include <list>
#include <algorithm>
#include <ext/hash_map>
#include "Pair.h"
#include "Cycle.h"
#include "Component.h"
#include "../multi/SortSeq.h"	// globalSortSequence
#include "../seqtools/seqfuncs.h"



//for use in classCycles():
struct CycleInteractions   //This keeps track of the cycles that interact
  {                        //  with this cycle.
  int comp;      //The component number this cycle is a part of.
  int min;       //The leftmost occurrence of this cycle.
  int max;       //The rightmost occurrence of this cycle.
  bool* tangled; //Those cycles that are entangled with this cycle.
  bool* within;  //within[i] == true  iff there is a node of cycle i
  };             //   between -min- and -max- in -cyclenums-.

//for use in findHurdles():
struct CompInteractions    //This keeps track of the components that interact
  {                        //  with this component.
  int min;         //The leftmost occurrence of this component.
  int max;         //The rightmost occurrence of this component.
  bool* separates; //within[i] == true  iff there is a node of component i
  };               //   between -min- and -max- in -compnums-.

struct Hurdle {		    // This keeps track of each hurdle
  bool isNull;		    // True if the hurdle values are nonsensical
  int comp;		    // Component number
  int min;		    // Leftmost node in the component
  int max;		    // Rightmost node in the component
			    //  (this is the rightmost node of the left half
			    //   of a maximal hurdle, since that's what's
			    //   needed for hurdle merging)
  Hurdle() {}		    // Default constructor
			    // Parameterized constructor
  Hurdle(bool i, int c, int mn, int mx)
    : isNull(i),
      comp(c),
      min(mn),
      max(mx) {}

  void setVars(int c, int mn, int mx) {comp=c; min=mn; max=mx;}
  bool operator<(const Hurdle h) const {return comp<h.comp;}
  bool operator==(const Hurdle h) const {return comp==h.comp;}
};

typedef __gnu_cxx::hash_map<int, Cycle> cyclemap;  //Map cycle label to cycle.


class CompInfo;
std::ostream& operator<<(std::ostream& output, CompInfo& c);

class CompInfo
  {
  friend std::ostream& operator<<(std::ostream& output, CompInfo& c);

  public:
      //Constructor is given the number of nodes in the graph.
    CompInfo(int nodes);
    CompInfo(const CompInfo& c);
    ~CompInfo();
    void operator=(const CompInfo& c);

      //Mark -node- as being part of cycle number -cyc- and having the
      //value -permval- in the permutation.
      // ** we assume that if -cyc- is higher than any cycle that currently
      //    exists, then it it one higher than the highest.
    void addToCycle(int node, int permval, int cyc);
      //Put the cycles into their respective components and mark the
      //components good (oriented) if one of it's constituent cycles is good.
    void classCycles();
      //Identify hurdles, superhurdles, fortress in all-zero-parity graph.
    void findHurdles();
      //Mark the component info as stale.
    void markStale();
      //Return true if the component info is stale.
    bool isStale() const;
      //Make the given cycle good.
    void setGood(int cycnum);

    void printCycle(int cyc);
    void printCycles();

      //Return the number of cycles over all components.
    int getNumCycles();
      //Return a vector of size of the cycles, sorted from largest to smallest.
    vector<int> getCycleSizes();
      //Return the size of the largest cycle.
    int getLargestCycleSize();
      //Return the number of components.
    int getNumComps();
      //Return the number of the cycle that -node- is a part of.
    int getCycleNum(int node) const;
      //Return the size of the cycle that -node- is a part of.
    int getCycleSize(int node);
      //Return a random node on the given cycle.
      // ** Do srandom((unsigned int)time(NULL)) first.
    int getRandomNodeOnCycle(int num);

      //Get a transposition that makes a cycle split into two,
      // otherwise return 0.
    int getTwoSplitTrans(ipair &interval);


      //Return the number of the component that -node- is a part of.
    int getCompNum(int node);
      //Get the number of bad (unoriented) components.
    int getBadCompCount();

      //Mark that the cycle containing this node has a deletion.
      //  ** Temporary.
    void setCycleDelVal(int node);
      //Return true if the given cycle is marked with deletions.
      //Reset the cycle's value.
      //  ** Temporary.
    bool getCycleDelVal(int cycle);

      //Merge the two cycles -c1- and -c2-.
      // ** only use this for doing a (r,d)-shrink (operate on a temp graph).
    void shrinkCycles(int node1, int node2);
      //Return the subset of oriented inversions from -set- that are on
      // component -c-.
    std::list<Pair> invsONcomp(int c, const std::list<Pair> &set) const;
      //Return the subset of oriented inversions from -set- that are on
      // the same component as all inversions in -invs-.
    std::list<Pair> invsONcomp(const std::list<Pair> &invs,
                               const std::list<Pair> &set) const;

      //Return the number of hurdles
    int getNumHurdles() {return hurdles.size();}
      //Return the number of simple hurdles
    int getNumSimHur() {return simHurdles.size();}
      //Return the number of superhurdles
    int getNumSupHur() {return superhurdles.size();}
      //Return the leftmost hurdle (simple or super)
    Hurdle get1stHurdle();
      //Return the next hurdle (simple or super)
    Hurdle getNextHurdle();
      //Return the leftmost simple hurdle
    Hurdle get1stSimHur();
      //Return the next simple hurdle
    Hurdle getNextSimHur();
      //Return the leftmost superhurdle
    Hurdle get1stSupHur();
      //Return the next superhurdle
    Hurdle getNextSupHur();

      //Return the node index of the max of the left half of the maximal hurdle
    int getMaxLM() {return maxLeftMax;}
      //Return the node index of the min of right half of the maximal hurdle
    int getMaxRM() {return maxRightMin;}
      //Return the node index of the max of the left half of the neo-hurdle
    int getNeoLM() {return neoLeftMax;}
      //Return true if the genome has a maximal hurdle
    bool hasMaximal() {return maximal;}
      //Return true if the genome is a fortress
    bool isFortress() {return fortress;}

    static const Hurdle nullHurdle; // Return this as a "null" hurdle


  private:
    void copy(const CompInfo& c);
    void del();

      //Add -cycle- to component -comp-.
    void addToComp(int cycle, int comp);
      //Build the component -compid- starting with cycle -cycleid- using the
      //  info in -cinfo-.
    void buildComp(int compid, int cycleid, CycleInteractions cinfo[]);
      //Return true if nodecycles is fully initialized.
    bool checkIfReady();
    bool checkNodecycles();
      //Merges simHurdles and superhurdles into hurdles
    void syncHurdles();




    cyclemap cycles;        //The map of Cycles.
    Component** components; //The array of pointers to Components.
    bool staleComp;         //Is true if the component info is out of date.

    int numnodes;           //The number of nodes in the graph.
    int numcycles;          //The number of cycles to be put into components.
    int numcomps;           //The number of components.

                            //These two are used for finding interactions:
    int* compnums;          //   The list of component numbers for the nodes.
    int* cyclenums;         //   The list of cycle numbers for the nodes.
    bool ready;             //TRUE if the cycles have been fully specified.

    int* cyccomp;           //cyccomp[i]-component that cycle i is a member of.

                            //The lists of hurdles:
    std::set<Hurdle> hurdles;      //Set of all hurdles
    std::set<Hurdle> simHurdles;   //Set of simple hurdles.
    std::set<Hurdle> superhurdles; //Set of superhurdles.

    bool maximal;	    //TRUE if maximal hurdle exists
    int maxLeftMax;	    //Max of left side of maximal hurdle
    int maxRightMin;	    //Min of right side of maximal hurdle
    bool fortress;	    //TRUE if genome is a fortress
    int neoLeftMax;	    //Min of left side of "neo-hurdle"
			    // (Neo-hurdle is a hurdle created by merging all
			    //  possible superhurdles in a fortress)

     std::set<Hurdle>::iterator
      currHurdle, 	    // Pointer to hurdles set
      currSimH,		    // Pointer to simHurdles set
      currSupH;		    // Pointer to superhurdles set
  };

#endif //COMPINFO_H
