/* Krister Swenson, Mark Marron                                   Spring 2003
The HP graph along with the overlap graph matrix from Anne Bergeron's paper
"A very elementary presentation of the Hannenhalli-Pevzner theory".

It uses Bergeron's overlap graph(matrix format) to compute the standard
inversions and the HP cycle graph to handle hurdles.
*/
#ifndef OGRAPH_H
#define OGRAPH_H

#ifdef _MSC_VER
#pragma warning (disable : 4503) // Turn off dumb Visual C++ warning
#endif

#include <iostream>
#include <sstream>
#include <string>
#include <cstdlib>
#include <cstdio>
#include <ctime>
#include <cmath>
#include <stack>
#include <vector>
#include <list>
#include <map>
#include <utility>
#include <algorithm>
#include <ext/hash_set>
#include "../multi/SortSeq.h"     // globalSortSequence
#include "CompInfo.h"
#include "Pair.h"
#include "../seqtools/seqfuncs.h"
#include "../tools/Graph.h"       //For finding a MIS of noninterfering revs.
#include "../tools/tools.h"       //For printing STL objects.
#include "../tools/trace/Trace.h" //For keeping trace normal forms.
#include "../tools/trace/Word.h"  //For keeping trace normal forms.

//#include "display/bpgraph.hpp" //For outputing the breakpoint graph in SVG.


               //Limit on exhaustive search for noninterfering inversions.
#define NINT_TOO_MANY_TO_SEARCH 200000

#define POS 1  //Possible values for the node sign.
#define NEG 0

using namespace std;

struct rEdge
  {
  int deletion;    //Marks whether there is a deletion.
  int copies;      //Marks whether there is a copy.
  int direction;   //Keeps track of the direction.
  int id;          //The index of the node connected to by this edge.
  };
struct dEdge
  {
  int id;          //The index of the node connected to by this edge.
  };

struct Node
  {
  int sign;             //POS for positive (used for consistancy checks).
  struct rEdge reality; //The reality edge.
  struct dEdge desire;  //The desire edge.
  int label;            //The node label.
  };

typedef pair<int,int> ipair;

class Ograph;
ostream& operator<<(ostream& output, Ograph& c);

class Ograph
  {                                         //Output the overlap matrix.
  friend ostream& operator<<(ostream& output, Ograph& c);

  public:

       //Build the graph. If given an invalid sequnce build the "0 1 2" graph.
    Ograph();
    Ograph(int len);  //Build the identity graph of length len.
    Ograph(const int sequence[], const int size, const bool invert=false);
    Ograph(const vector<int> sequence, const bool invert=false);
    Ograph(const Ograph& g);
    ~Ograph();
    void operator=(const Ograph& g);

      //Return true if the given graph is equivalent to -this-.
    bool operator==(const Ograph& g) const;

    bool notSorted() const;
      //Return true if this is a positive permutation.
    bool isPositive() const;
      //Do all reversals necessary to make this graph into a bunch of two node
      //cycles,  including hurdle cutting and merging.  Return the inversion
      //distance.  Fill the given vector with the inversions done.
    int resolve(vector<Pair> &revs);
      //Mark the given values as deletions.
    int markDels(stack<Pair> &dels);

      //Return the inversion distance WITHOUT CONSIDERING HURDLES from this
      //sequence to the identity.
      // ** No inversions are actually done (constant time op).
    int getDistNH() const;
      //Return the inversion distance from this sequence to the identity.
      //  ** No inversions are actually done.
      //  ** THIS runs resolve() on a copy of the graph BECAUSE findHurdles
      //     (in CompInfo) only works on positive permutations.
    int getDist() const;

    //~%~%~%~%~%~%~%~%~%~
    //The indices for the inversions in the following are 1-indexed and
    //inclusive.  i.e. inversion work on interval [x,y]...
    //    for example rev(1,1) just reverses the first element of the sequence.
      //Reverse -seq- and -seqinds- over the given interval.
      //Take sequence indices.
    void rev(int min, int max);
      //Reverse -seq-, -seqinds-, and -gr- over the given interval.
      //Take sequence indices. Don't update the component info (for efficiency).
    void revAllNoUpdate(int min, int max);
      //Reverse -seq-, -seqinds-, and -gr- over the given interval.
      //Take sequence indices. Update the component info as well.
    void revAll(int min, int max);
      //Reverse -seq-, -seqinds-, and -gr- over the given interval.
      //  Take a pair. Update the component info as well.
    void revAll(const Pair& p);
    void revAll(const ipair& p);
      //Do the given reversal as though it were done on the inverse graph...
      //  in other words, do an inversion on the identity and make the
      //  appropriate changes to the breakpoint graph.
      //  Update everything.
    void revAllInv(const Pair& p);
      //Return true if the given pair represents a safe reversal on the
      //current graph.
      //  ** see the disclaimer in the Ograph.C.
    bool isSafe(const ipair& p) const;
    bool isSafe(const Pair& p) const;
      // Return true if an adjacency is created by the given reversal.
    bool createsAdjacency(const ipair r) const;
      //Apply the given set of commuting inversions to the permutation.
      //If -force- is false, stop and return a pointer to an unsafe inversion
      //in -set- if one exists.
      //Set is not modified.
    list<Pair>::iterator applyCommutingSet(list<Pair> &set, bool force=false);
      //Re-apply the given set of commuting inversions to the permutation.
      // (the indices in the -set- are valid only if applied from longest to
      //  shortest -- this undoes the action done by applyCommutingSet())
    void reApplyCommutingSet(const list<Pair> &set);
    //~%~%~%~%~%~%~%~%~%~


      //Return a copy of the sequence without the frame.
      //  ** Caller responsible for the memory.
    int* getSeq() const;
    vector<int> getSeqVec() const;
      //Return the sequence length without the frame elements (0 and n+1).
      //  This is the size of the array returned in getSeq().
    int getSeqLen() const;
      //Return an array of the sequence (without the frame) named according to
      //the given sequence.  The given sequence must be the length of this
      //sequence without the frame elements 0 and n-1.
      //  ** Caller responsible for the memory.
    int* getRenSeq(const int s2[], int s2len) const;
      //Return a character array of the sequence without the frame. Caller
      //  responsible for the memory.
    char* getSeqStr() const;
      //Return a string of the sequence without the frame.
    string getSeqString() const;
      //Return a string of the sequence (without the frame) named according to
      //  the given sequence.  The given sequence must be the length of this
      //  sequence without the frame elements (0, n-1).
      //  ** Caller responsible for the memory.
    char* getRenStr(const int s2[], int s2len) const;
      //Return a string of the inverse of the sequence (without the frame).
      //  ** Caller responsible for the memory.
    char* getInvStr() const;
      //Return the inverse graph of the current one.
    Ograph getInvGraph() const;


    //~%~%~%~%~%~%~%~%~%~
    //The inversions in the following are 1-indexed:
      //Fill a vector with all pairs of divergent reality edges that represent
      //safe inversions.  These pairs indicate the inversions that can be done
      //to decrease the inversion distance.
      //Return the size of the vector.
      //  ** This isn't the complete list of sorting inversion if there are
      //     hurdles around.
    int getSafeInvs(vector<Pair> &ops) const;
      //Fill a vector with all pairs of divergent reality edges that represent
      //cycle-splitting inversions.
      //Return the size of the vector.
      //  ** This isn't the complete list of sorting inversion if there are
      //     hurdles around.
      //  ** Unsafe inversions are included in this list.
    int getCSInvs(vector<Pair> &ops) const;
      //Return a vector of all pairs of divergent reality edges (with one of
      //the endpoints being that which is given) that represent cycle splitting
      //inversions.  These pairs indicate the inversions that can be done
      //to decrease the inversion distance.
      //  ** This isn't the complete list of sorting inversion if there are
      //     hurdles around.
    void getCSInvs(vector<Pair> &ops, const int endpoint) const;
      //Fill a vector with all oriented (adjacency creating) inversions.
      //Return the size of the vector.
      //  ** Unsafe inversions are included in this list.
    int getOrientedInvs(vector<Pair> &ops) const;
    //~%~%~%~%~%~%~%~%~%~

      //Return a vector with an entry per (large enough) cycle.  Each entry
      // contains the ratio of right-facing reality edges to the size of the
      // cycle.
    vector<float> getOrientationBalance() const;
      //Return the ratio of right-facing reality edges to the size of the cycle
      // for the largest cycle.
    float getOrientationBalanceForLargest() const;
      //Return the length of the largest cycle.
    int getLargestCycleSize() const;
      //Return the length of the cycles, sorted from largest to smallest.
    vector<int> getCycleSizes() const;
      //Return a vector of all pairs of non-divergent reality edges from the
      //same cycle.  These pairs indicate the inversions that can be done that
      //have no effect on the inversion distance.
   void getBenignInvs(vector<Pair> &ops) const;
      //Return a list of all pairs of reality edges that are on different
      //cycles.
    void getUnsortingInvs(vector<Pair> &ops) const;
    //~%~%~%~%~%~%~%~%~%~



      //Return a maximum independent set of commuting inversions.
    list<Pair> getCommutingMIS() const;
      //Recursively print the normal forms that represent all sorting paths.
    //void printTraces(vector< list<Element> > &nf);
      //Reset the datastructures that hold info about maximal independent sets.
    void resetMalISinfo();
      //Return the next maximal independent set of noninterfering inversions.
    //list<Pair> getNextNintMalIS();
      //Return a vector of all maximal safe first words in a normal form for
      //this graph.
    vector< list<Pair> > getMalWords(bool onlysafe=true);
      //Return a vector of all maximal safe first words in a normal form for
      //this graph.
    vector<Word> getMalWordsW(bool onlysafe=true);
      //Print the traces that correspond to maximal safe words.
    void printMalWordTraces(ostream &out);
      //Print the traces corresponding to maximal safe words starting from the
      //inverse permutation.
    void printInvMalWordTraces(ostream &out);
      //Put in -retval- the traces that correspond to maximal safe words.
      // ** assume the graph's state corresponds to the given trace.
    void getMalWordTraces(Trace &trace, std::vector<Trace> &retval);
      //Return a vector of all safe first revs in a normal form that don't
      //commute with the given word.
    vector< list<Pair> > getWords(const list<Element> &word=list<Element>());
      //Return a vector of all safe first words in a normal form for
      //this graph.
    vector<Word> getWordsW();
      //Print the traces that correspond to this permutation.
    void printWordTraces(ostream &out);
      //Print the traces starting from the inverse permutation by getting one
      //word at a time.
    void printFromInvWordTraces(ostream &out);
      //Put in -retval- all traces corresponding to this permutation.
      // ** assume the graph's state corresponds to the given trace.
    void getWordTraces(Trace &trace, std::vector<Trace> &retval,
                       const list<Element> &prevword=list<Element>());
      //Print the traces that correspond to this permutation.
      //Do it by only visiting prefixes that correspond to appending scenarios.
    void printAppendingTraces(ostream &out);
      //Return all traces corresponding to this permutation.
    vector<Trace> getAppendingTraces();


      //Print the words that come from independent sets of noninterfering
      //inversions
    void printWords(ostream& out);
      //Print the words that come from maximal independent sets of
      //noninterfering inversions.
    void printMalWords(ostream& out, bool onlysafe=true);
      //Return a maximum independent set of noninterfering inversions.
    list<Pair> getNonintMIS() const;
      //Return a mutual maximum independent set of noninterfering inversions.
    list<Pair> getMutualNonintMIS(const Ograph &hpg) const;
      //Return true if the given set is noninterfering.
    bool isNonint(list<Pair> set) const;
      //Get the graph represening the interference constraints.
    Graph getNonintGraph() const;
      //Print the graph represening the interference constraints.
    void printNonintGraph() const;
      //Return set.end() if the give commuting set is noninterfering.
      //Otherwise return a pointer to an offending inversion in -set-.
      //-set- is not modified.
    list<Pair>::iterator findInterfering(list<Pair> &set) const;
      //Return true if the given set of inversions commutes.
    bool isCommuting(const list<Pair> &set) const;
      //Return the chord model for the commuting constraints.
      //Fill -intToinv- with the map from chord to inversion Pairs.
    vector<int> getCommutingCM(vector<ipair> &intToinv) const;
      //Return the chord model for the sorting constraints labeled based on the
      // given commuting chord model and chord to inversion map -chordTOinv-.
    vector<int> getSortingCM(const vector<int> &ccm,
                             const vector<ipair> &chordTOinv) const;

      //True if the union of commuting and noninterfering constraints can be
      // represented by a chord model.
    bool isCG() const;
      //Return a permutation that is a shrink of the current one where each
      // component is a single cycle.
    vector<int> getShrink(__gnu_cxx::hash_set<int> &removed) const;

      //True if the given Pair represents a reality edge on different cycles.
    //bool isUnsortingInv(Pair &op) const;

      //Convert a list of inversions in endpoint format to a list of inversions
      //in set format.
    list< set<int> > invsTOsets(const list<Pair> invs) const;
      //Convert the inversion in endpoint format to an inversion in set format.
    set<int> invTOset(const Pair &inv) const;

      //Convert a vector of inversions in endpoint format to a vector of
      //inversions in set string format.
    vector<std::string> invTOsetstr(const vector<Pair> invs) const;
    vector<std::string> invTOsetstr(const list<Pair> invs) const;
    vector<std::string> invTOsetstr(const vector<ipair> invs) const;

      //Print the given normal form.
    static void printNF(ostream& out, const vector< list<Element> > nf);
      //Print a list of sets that represent inversions.
    static void printInvSet(ostream& output, const list<Element> slist);


    int getGrSize() const;
    int getMatSize() const;
    void printGraph() const;             //Output the graph.
    void printGrInfo() const;            //Print info on seq and gr (for DEBUG).
    void printSeqStr() const;            //Print the sequence to stderr.
      //Print the sequence to stderr with '*' to indicate FORWARDS
      //oriented edges.
    void printOrientationStr() const;
    void printCycles() const;            //Print the cycles.
    void printCompInfo() const;          //Print the component information.
    //void printGraphSVG(ostream&) const;  //Output the graph in SVG format.



    //~%~%~%~%~%~%~%~%~%~
    // TRANSPOSITIONS:
      //Do the specified transposition. Elements are 1 indexed.
      //  ** We just annihilate everything and rebuild the object based on our
      //     calculated new sequence.
    void doTrans(int left, int right, int before);
      //Do a transposition that splits two new cycles, otherwise return false.
    bool doTwoSplitTrans();
      //Do a transposition that joins three cycles, otherwise return false.
      // ** Do srandom((unsigned int)time(NULL)) first.
    bool doThreeMergeTrans();



  private:
    void construct(const int sequence[], const int size,
                   const bool invert=false);
    void copy(const Ograph& g);
    void del();
      //Fill -comps- with information for all the cycles of the graph 
      //including the cycle numbers as well as a mark on the cycles that
      //are good (oriented).
    void findCycles();
      //Update the information about the cycles, components, and -gr-.
    void updateCompInfo();
    void findGraphArray();	  //Build the -gr- array 
    void findOverlapMat();        //Build the overlap matrix for this graph.
    void findOvMatHaveMem();	  //Build matrix (mem already allocated) 

    bool* getAdjArray(int index); //Get the adjacency array for a vertex.
    bool isOriented(int vertex);  //Determine if a vertex is oriented.
    int compScore(int vertex);    //Compute the score for a vertex.
    int getMaxScoreVert();        //Get the vertex index with the maximal score.
              //Adjust the score array given vertex v was changed.
    void adjScoreArray(int maxScoreVert);
              //Adjust the adjacency lists when a vertex -vert- is changed.
    void adjAdjList(int vert);
              //Adjust the adj stuff for the w in the paper.
    void adjForW(int wInd, bool *w, bool *v, int gainLoss);
    void adjParity(int v);        //Adjust the parity.
              //Adjust the adjacency list after an unoriented reversal.
    void adjAdjList(vector<int> nodes);
              //Adjust the parity after an unoriented reversal.
    void adjParity(vector<int> nodes);

              //Get rid of all hurdles. (Joel)
    int resolveHurdles(vector<Pair> &revs);
              //Do 1st cut/merge of graph with odd # of hurdles
    bool doFirstCut(vector<Pair> &revs, int &hurdleToSkip);
              //Return the left index of the first hurdle-merging reversal 
    int getFirstLeft(int &hurdleToSkip);
              //Merge the next two hurdles (skip hurdle if necessary)
    bool mergeNextTwo(vector<Pair> &revs, int &left, int &hurdleToSkip);
              //Merge the next two hurdles
    bool mergeNextTwo(vector<Pair> &revs, int &left);
              //Cut the maximum hurdle 
    void cutMaxHurdle(vector<Pair> &revs);
    Pair doRev();   //Do the next reversal. Return interval(inclusive) reversed.
              //Do an unoriented reversal and return interval(inclusive).
    Pair doUnorRev(int left, int right);
    bool allZeroParity(); //Is the parity 0 for all the verts.
    bool allZeroScore();  //Is the score 0 for all the verts.(There are hurdles)
    int getNumCycles() const; //Return the number of cycles over all components.
              //Return the number of hurdles.
              //  precondition: the sequence is a positive permutation.
    int getNumHurdles() const;
    void printArray(const int a[], int seqsize) const;
              //Sort the elements by the position they occur in the sequence.
    void sortByPosition(list<int> &v) const;
              //Merge lists based on position of the element in the sequence.
    void merge(list<int> &l, list<int> & l2) const;

              //Try to find a noninterfering independent set of size k.
              //Return true and modify -is- if we find one.
              //  ** tries all n choose k subsets of size k!
    bool findNISofsizek(int k, const Graph &g, list<int> &is,
                               const vector<ipair> &chordTOinv,
                               const Ograph* og = NULL) const;
              //Recursively try to find an independent set by adding -k- more
              // elements from -from- to -set-.  When k=0, find the IS of set.
              // If a set if found true is returned and -set- contains the set.
    bool tryISofsizek(int k, list<Pair> &set, list<Pair>::iterator from,
                             const list<Pair>::const_iterator &end,
                             const Ograph* og) const;

              //Convert a chord list to its corresponding Pair (inversion)
              //list using the map defined by chorTOinv.
    static list<Pair> cTOinv(const list<int> &set,
                             const vector<ipair> &chordTOinv);
              //Convert a chord list to its corresponding set (inversion)
              //using the map defined by chorTOinv.
    list< set<int> > cTOset(const list<int> &set,
                            const vector<ipair> &chordTOinv) const;
              //Convert a Pair (inversion) list into a chord list.
              //The mapping is defined by -invTOchord-.
    static list<int> invTOc(const list<Pair> &l,
                            const map<ipair,int> &invTOchord);
              //Invert the int to ipair mapping defined by -chordTOinv-.
    static map<ipair,int> invertMap(const vector<ipair> &chordTOinv);


              //Try to find an independent set one samller than -is- that
              // doesn't include at least 1 from -is-.
              //Return true and modify -is- if we find one that
              // is noninterfering for this and graph og.
    bool tryISwithout(const Graph &g, list<int> &is,
                      const vector<ipair> &chordTOinv,
                      const Ograph* og = NULL) const;
              //Remove the unsafe inversions from the given chord model.
    void removeUnsafe(vector<int> &cm, const vector<ipair> &chordTOinv) const;

              //Restrict the given chord models to have only the mutual
              // inversions.
              //Return the mutual chord to inversion (ipair) mapping.
    static vector<ipair> restrictTOcommon(vector<int> &c1, vector<int> &s1,
                                          const vector<ipair> &cTOi1,
                                          vector<int> &c2, vector<int> &s2,
                                          const vector<ipair> &cTOi2);
              //Return true if there is a nonzero intersection.
    static bool shareInvs(const list<Pair> &s1, const list<Pair> &s2);


      //Return true if the given word is a complete first word of a trace
      //for this permutation.
    bool hasInterferencePath(const Word &winvs, std::list<Pair> &pinvs,
                             const std::list<int> &chords);
      //Return the next safe inversions that don't enter the first word of the
      //given normal form.
    std::list<Pair> getNoncommSet(const Trace &t);
      //Return the next safe inversions that would be the last element of a
      //word other than the first word if added to the normal form -t-.
    std::list<Pair> getExtensionSet(const Trace &t);
      //Return the next safe inversions that would be the last element if
      //added to the normal form -t-.
    std::list<Pair> getAppendingSet(const Trace &t);
    void getAppendingTraces(vector<Trace>& retval, Trace& t);

      //Return true if there exist no sequence of reversals that enters the
      //first word of -t-.
    bool hasInterferencePath(Trace &t);

      //Return true if the given Element commutes with the given list
      // of Elements.
    static bool commutesWith(const list<Element> &slist, const Element &s);
      //Return true if the given Pair commutes with the given list of pairs.
    static bool commutesWith(const list<Pair> &plist, const Pair &p);
      //Return true if the two sets intersect (one is not the subset
      // of the other)
    static bool intersect(const set<int> &s1, const set<int> &s2);
      //Return true if the two intervals intersect (one is not the subset
      // of the other)
    static bool intersect(const Pair &p1, const Pair &p2);





              //Return a string from the given input.
    template < class T >
    string ToString(const T &arg) const;

              //Return true if the specified transposition will create two
              // more cycles.
    bool isTwoSplitTrans(int left, int right);





    int graphsize;     //The number of nodes in the graph.
    CompInfo* comps;   //The info about the components.
    struct Node* gr;   //The array of nodes (indexed by place in graph).
                       //  ** Maintained through reversals for revAll() only!
                       //  ** Mabrouk deletion mechanism is calculated
                       //     through the number of cycles.

    int* seq;          //The sequence. This is maintained through reversals.
    int seqsize;       //The size of the sequence with bookends 0 and n+1.
    int* seqinds;      //seqinds[i] -> index into -seq- array for val abs(i).

                       //Bergeron values:
    bool** matrix;     //The Overlap Matrix -> holds edge adjacencies.
    bool* p;           //The parity.
    int* s;            //The score.
    int matsize;       //The width of the matrix.
    vector<bool> inRev;//True for each node in an unoriented reversal.

    Graph nintcon;     //The graph representing the noninterfering constraints.
    vector<ipair> cTOi;//Chord to inversion map for the nintcon graph.
  };


#endif
