/*  Krister Swenson,Mark Marron                                     Fall 2002
The HP graph along with the overlap graph matrix of Anne Bergeron's paper 
"A very elementary presentation of the hannenhalli-pevzner theory".
*/

#include "Ograph.h"

using namespace std; 


/* Create a graph that represents the given -sequence-.
  ** If given an invalid sequence then just build the graph as though we were
     given the sequence "1".
*/
Ograph::Ograph(const int sequence[], const int size, const bool invert)
  { construct(sequence, size, invert); }
Ograph::Ograph(const vector<int> sequence, const bool invert)
  {
  int temp[sequence.size()];
  for(unsigned i=0; i < sequence.size(); i++)
    temp[i] = sequence[i];
  construct(temp, sequence.size(), invert);
  }

void Ograph::construct(const int sequence[], const int size, const bool inv)
  {
  if(validPerm(sequence, size))  //If we're given a valid permutation
    {                            //then set up the sequence:
    seqsize = size + 2;
    graphsize = size*2 + 2;
    matsize = graphsize/2;


    seq = new int[seqsize];     //Set up the sequence:
    seqinds = new int[seqsize];
    seq[0] = 0;
    seqinds[0] = 0;
                  //We've removed Joel's invert sequence thing.  Hopefully his
                  //Sorting sequences still turn out OK (just inverted).
    if(inv)                     //If we want to build the inveted sequence:
      {
      int gene;
      for(int i=1; i <= size; i++)//Invert sequence so seq is genome 1 from an
        {                         //identity-permutation-numbered genome 2. Then
        gene = sequence[i-1];     // the reversal sequence will convert genome 1
        if (gene >=0)             // into genome 2.
          {                       //  `--Joel Added this, not sure why?
          seq[gene] = i;
          seqinds[i] = gene;
          }
        else
          {
          seq[-gene] = -i;
          seqinds[i] = -gene;
          }
        }
      }
    else
      {
      for(int i=1; i <= size; i++)
        {
        seq[i] = sequence[i-1];
        seqinds[abs(seq[i])] = i;
        }
      }
    seq[seqsize-1] = seqsize-1;
    seqinds[seqsize-1] = seqsize-1;
    }
  else                           //Otherwise, just make the sequence be "0 1 2":
    {
    cerr << "(Ograph)Warning: Invalid sequence given. Creating default.\n";
    seqsize = 3;
    graphsize = 4;
    matsize = 2;
    seq = new int[3];
    seqinds = new int[3];
    seq[0]=0;seq[1]=1;seq[2]=2;
    seqinds[0]=0;seqinds[1]=1;seqinds[2]=2;
    }


  gr = (struct Node*)malloc(graphsize * sizeof(struct Node));
  findGraphArray();
  findOverlapMat();
  comps = 0;           //Initialize.
  findCycles();        //Find the cycles(and components) in the initial graph.
  }

/* Copy constructor.
*/
Ograph::Ograph(const Ograph& g)
  { copy(g); }

/* Destructor
*/
Ograph::~Ograph()
  { del(); }

/* Delete the memory for this.
*/
void Ograph::del()
  {
  free(gr);
  
  delete comps;
  
  for(int i=0; i < matsize; i++)
    delete [] matrix[i];      //Delete the rows.
  free(matrix);               //Free the columns.

  delete [] p;
  delete [] s;

  delete [] seq;
  delete [] seqinds;
  }


/* Assignment Operator.
*/
void Ograph::operator=(const Ograph& g)
  { del(); copy(g); }

/* Copy the memory from the object.
*/
void Ograph::copy(const Ograph& g)
  {
  graphsize = g.graphsize;
  gr = (struct Node*)malloc(graphsize * sizeof(struct Node));
  for(int i=0; i < graphsize; i++)
    {
    gr[i].sign = g.gr[i].sign;
    gr[i].reality = g.gr[i].reality;
    gr[i].desire = g.gr[i].desire;
    gr[i].label = g.gr[i].label;
    }

  comps = new CompInfo(g.graphsize);
  *comps = *(g.comps);

  seqsize = g.seqsize;
  seq = new int[seqsize];
  seqinds = new int[seqsize];
  for(int i=0; i < seqsize; i++)
    {
    seq[i] = g.seq[i];
    seqinds[i] = g.seqinds[i];
    }

  matsize = g.matsize;
  matrix = (bool **)malloc(matsize*sizeof(bool*));
  for(int i=0; i < matsize; i++)
    {
    matrix[i] = new bool[matsize];
    for(int j=0; j < matsize; j++)
      matrix[i][j] = g.matrix[i][j];
    }

  p = new bool[matsize];
  s = new int[matsize];
  for(int i=0; i < matsize; i++)
    {
    p[i] = g.p[i];
    s[i] = g.s[i];
    }

  inRev = g.inRev;
  }

int Ograph::getGrSize() const
  { return graphsize; }

int Ograph::getMatSize() const
  { return matsize; }


/* Return true if the sequence is not sorted.
*/
bool Ograph::notSorted() const
  { return  (comps->getNumCycles() != seqsize-1); }


/*Return true if this is a positive permutation.
*/
bool Ograph::isPositive() const
  {
  for(int i=0; i<seqsize; i++)
     {
     if(seq[i] < 0)
       return false;
     }

  return true;
  }


/* Fill -gr- according to the contents of -seq-.
*/
void Ograph::findGraphArray() {
  int i;
  int size = seqsize - 2;
  vector<int> 
    nodeLoc(graphsize);	      //Location of node i in the graph:0,-1,1,-2,2... 

  nodeLoc[0] = 0;             //Initialize the frame(two added nodes):
  nodeLoc[graphsize-1] = graphsize-1;

  gr[0].sign = 0;             //sign is meaningless for the first node.
  gr[0].label = 0;            //my addition
  gr[0].reality.id = 1;
  for(i=1; i <= size; i++)
    {
    if(seq[i] > 0)
      {
      nodeLoc[abs(seq[i]*2)] = 2*i;
      nodeLoc[abs(seq[i]*2)-1] = (2*i)-1;

      gr[(2*i)-1].sign = NEG;
      gr[2*i].sign = POS;
      }
    else
      {
      nodeLoc[abs(seq[i]*2)-1] = 2*i;
      nodeLoc[abs(seq[i]*2)] = (2*i)-1;

      gr[(2*i)-1].sign = POS;
      gr[2*i].sign = NEG;
      }

    //gr[(2*i)-1].label = -sequence[i-1];   //THIS IS SetubalMed CONVENTION:
    //gr[2*i].label = sequence[i-1];
    if(seq[i] < 0)		           //THIS IS Bergeron CONVENTION:
      {
      gr[(2*i)-1].label = -2*seq[i];
      gr[2*i].label = (-2*seq[i])-1;
      }
    else
      {
      gr[(2*i)-1].label = (2*seq[i])-1;
      gr[2*i].label = 2*seq[i];
      }

    gr[(2*i)-1].reality.id = 2*(i-1); 
    gr[2*i].reality.id = (2*(i+1))-1; 
    }
  gr[graphsize-1].sign = 0;   //sign is meaningless for the last node.
  gr[graphsize-1].reality.id = graphsize-2;
  gr[graphsize-1].label = graphsize-1;


  gr[0].desire.id = nodeLoc[1];                      //The first.
  gr[graphsize-1].desire.id = nodeLoc[graphsize-2];  //The last.
  for(i=1; i <= size; i++)
    {
    gr[nodeLoc[(i*2)-1]].desire.id = nodeLoc[(i*2)-2];
    gr[nodeLoc[i*2]].desire.id = nodeLoc[(i*2)+1];
    }
}


// Different values for -direction- in findCycles() :
#define FORWARDS 1
#define BACKWARDS 0
#define NEITHER 2
// Different values for -nextedge- in findCycles() :
#define REALITY 0
#define DESIRE 1

/* Fill -comps- with information for all the cycles of the graph including
the cycle numbers as well as a mark on the cycles that are good (oriented).
Mark the component info as being stale (i.e. comps->classCycles() must be 
called to update the component info if desired).
*/
void Ograph::findCycles()
  {
			      //True for nodes that are not yet part of a cycle.
  vector<bool> unchecked(graphsize);
  for(int i=0; i < graphsize; i++)          //Init:
    unchecked[i] = true;

  int start = 0;              //The spot to start looking for a cycle at.
  int cyclenum = 0;           //The current cycle we're looking at.
  int nextedge = REALITY;     //The next edge-type to be checked.
  if(comps)
    delete comps;
  comps = new CompInfo(graphsize);
  while(start < graphsize)
    {
    int directn = NEITHER;    //Cycle direction induced by the current edge.
    int current = start;      //We're looking at the edge coming from this node.
    while(unchecked[current])
      {                                       //Add this node to the cycle.
      comps->addToCycle(current, abs(seq[(int)ceil(current/2.0)]), cyclenum);
      unchecked[current] = false;             //Mark this node as checked.

      if(nextedge == REALITY)      //For reality edges we need to
        {                          //worry about the goodness of the cycle:
        if(directn == NEITHER)                //This is the first reality edge
          {
          if(gr[current].reality.id > current)// so we set the direction:
            {
            gr[gr[current].reality.id].reality.direction =
            gr[current].reality.direction = (directn = FORWARDS);
            }
          else
            {
            gr[gr[current].reality.id].reality.direction =
            gr[current].reality.direction = (directn = BACKWARDS);
            }
          }
        else if(directn == FORWARDS)          //If cycle direction is forwards
          {
          if(gr[current].reality.id < current)// and current edge is backwards
            {
            comps->setGood(cyclenum);         // we know the cycle is good.
            gr[gr[current].reality.id].reality.direction =
            gr[current].reality.direction = BACKWARDS;
            }
          else
            {
            gr[gr[current].reality.id].reality.direction =
            gr[current].reality.direction = FORWARDS;
            }
          }
        else if(directn == BACKWARDS)         //The cycle direction is backwards
          {
          if(gr[current].reality.id > current)// and current edge goes forward
            {
            comps->setGood(cyclenum);         // so we know the cycle is good.
            gr[gr[current].reality.id].reality.direction =
            gr[current].reality.direction = FORWARDS;
            }
          else
            {
            gr[gr[current].reality.id].reality.direction =
            gr[current].reality.direction = BACKWARDS;
            }
          }
        else
          cerr << "\n++++ We have a bad direction value\n";

        nextedge = DESIRE;         //The next edge will be a desire edge.
        current = gr[current].reality.id;     //Update current to the next node.
        }
      else                         //For desire eges,
        {
        nextedge = REALITY;              //the next edge will be a reality edge.
        current = gr[current].desire.id; //Update -current- to the next node.
        }
      }
    cyclenum++;
                                   //Find the next start spot:
    while(!unchecked[start] && (++start < graphsize));
    }
  comps->markStale();
  }

/*Return the number of hurdles over all components.
  ** Precondition: the sequence is a positive permutation!
*/
int Ograph::getNumHurdles() const
  { return comps->getNumHurdles(); }

/*Return the number of cycles over all components.
*/
int Ograph::getNumCycles() const
  { return comps->getNumCycles(); }


/* Mark the given values as deletions.  Return the number of operations
needed to delete them.
** For now just mark the edge as containing at least one deletion.
   `-> This is a simplified approximation of El-Mabrook's method.
       (if hurdles exist, it's not completely accurate [0 or 1 del per merge])
*/
int Ograph::markDels(stack<Pair> &dels)
  {
  while(!dels.empty())
    {
    //cerr << "DEL:" << dels.top() << " | " << 2*dels.top().getRight() << endl;
    //cerr << "  seqinds:" << gr[seqinds[abs(dels.top().getRight())]*2].label
    //     << endl;
    int node = abs(dels.top().getRight());
    gr[seqinds[node]*2].reality.deletion = dels.top().getLeft();
    comps->setCycleDelVal(node*2);
    dels.pop();
    }

  int numdels = 0;                 //For each cycle in the graph,
  for(int i=0; i < comps->getNumCycles(); i++)
    if(comps->getCycleDelVal(i))   //if the cycle has a deletion, then consider
      numdels++;                   //all the dels in the cycle to be 1.

  return numdels;
  }



/* Return a copy of the sequence without the frame.
Caller is responsible for the memory.
*/
int* Ograph::getSeq() const
  {
  int i;
  int* ret = (int*)calloc(seqsize-2, sizeof(int));

  for(i=0; i<seqsize-2; i++)
    ret[i] = seq[i+1];

  return ret;
  }

/*Return an array of the sequence (without the frame) named according to
the given sequence.  The given sequence must be the length of this sequence
without the frame elements 0 and n-1.
  ** Caller responsible for the memory.
*/
int* Ograph::getRenSeq(const int s2[], int s2len) const
  {
  int *s1 = &seq[1];         //Make s1 so that it points to the second element
                             // of seq (ignor the 0 at the beginining).
  int s1len = seqsize-2;     //The length of seq without the frame elements.
  int* ret = (int*)calloc(s1len, sizeof(int));

  assert(s1len == s2len);    //Ensure the sequence lengths match.

  int index[s2len+1];        //index[i] is the index where i is located in s2.
                             // (so should be the value that i becomes in s1)
  for(int i=0; i<s2len; i++)     //Build the index:
    {
    if(s2[i] < 0)                //If the number is negative
      index[abs(s2[i])] = -(i+1);//then store its index as negative
    else                         //otherwise,
      index[s2[i]] = (i+1);      //store it as positive.
    }

  for(int j=0; j<s1len; j++)     //Use the index.
    {
    if(s1[j]>0)
      ret[j] = index[s1[j]];
    else
      ret[j] = -index[abs(s1[j])];
    }

  return ret;
  }


/* Return the sequence length without the frame elements (0 and n+1).
This is the size of the array returned in getSeq().
*/
int Ograph::getSeqLen() const
  { return seqsize-2; }

/* Return a character array of the sequence without the frame.
Caller is responsible for the memory.
*/
char* Ograph::getSeqStr() const
  {
  string s = getSeqString();
  char* ret = (char*)calloc(s.length()+1, sizeof(char));
  s.copy(ret, s.length());
  ret[s.length()] = '\0';

  return ret;
  }

/* Return a string of the sequence without the frame.
*/
string Ograph::getSeqString() const
  {
  string ret = ToString(seq[1]);

  for(int i=2; i<seqsize-1; i++)
    ret += " " + ToString(seq[i]);

  return ret;
  }


/*Return a string of the sequence (without the frame) named according to
the given sequence.  The given sequence must be the length of this sequence
without the frame elements (0, n-1).
  ** Caller responsible for the memory.
*/
char* Ograph::getRenStr(const int s2[], int s2len) const
  {
  int *s1 = &seq[1];         //Make s1 so that it points to the second element
                             // of seq (ignor the 0 at the beginining).
  int s1len = seqsize-2;     //The length of seq without the frame elements.

  assert(s1len == s2len);    //Ensure the sequence lengths match.

  int index[s2len];          //index[i] is the index where i is located in s2.
                             // (so should be the value that i becomes in s1)
  for(int i=0; i<s2len; i++)     //Build the index:
    {
    if(s2[i] < 0)                //If the number is negative
      index[abs(s2[i])] = -(i+1);//then store its index as negative
    else                         //otherwise,
      index[s2[i]] = i+1;        //store it as positive.
    }

  char* ret = (char*)calloc(seqsize*3, sizeof(char));
  ret[0] = '\0';
  for(int j=0; j<s1len; j++)     //Use the index.
    {
    if(s1[j]>0)                  //If this element is positive then the sign
      sprintf(ret, "%s %i", ret, index[abs(s1[j])]);  //depends on the index.
    else                         //otherwise,
      sprintf(ret, "%s %i", ret, -index[abs(s1[j])]); //it's the inverse.
    }

  return ret;
  }

/* Return a string of the inverse of the sequence (without the frame).
  ** Caller responsible for the memory.
*/
char* Ograph::getInvStr() const
  {
  int ts[seqsize-2];

  for(int i=0; i<seqsize-2; i++)
    {
    if(seq[i+1] > 0)
      ts[abs(seq[i+1])-1] = i+1;
    else
      ts[abs(seq[i+1])-1] = -(i+1);
    }


  char* ret = (char*)calloc(seqsize*3, sizeof(char));
  ret[0] = '\0';
  for(int i=0; i<seqsize-2; i++)
    sprintf(ret, "%s %i", ret, ts[i]);  //depends on the index.

  return ret;
  }

/* Return the inverse graph of the current one.
*/
Ograph Ograph::getInvGraph() const
  {
  int inv[seqsize-2];
  for(int i=1; i<seqsize-1; i++)  //Copy seq to inv.
    inv[i-1] = seq[i];
  invertSeq(inv, seqsize-2);      //Invert inv.

  Ograph ret(inv, seqsize-2);     //Build the graph
  return ret;                     // that we return.
  }


/* Return the inversion distance WITHOUT CONSIDERING HURDLES from this sequence
to the identity.
  ** No inversions are actually done.
*/
int Ograph::getDistNH() const 
  {
  return (seqsize-1) - getNumCycles();
  }

/* Return the inversion distance from this sequence to the identity.
  ** No inversions are actually done.
  ** THIS runs resolve() on a copy of the graph BECAUSE findHurdles
     (in CompInfo) only works on positive permutations.
*/
int Ograph::getDist() const 
  {
  Ograph tmp = *this;
  vector<Pair> garbage;
  int dist = tmp.resolve(garbage);

  // return ((seqsize-1) - getNumCycles()) + ((getNumHurdles()+1)/2);
  return dist;
  }


/* Do all reversals necessary to make this graph into a bunch of two node
cycles, including hurdle cutting and merging.  Return the inversion distance.
Fill the given vector with the inversions done.
*/
int Ograph::resolve(vector<Pair> &revs)
  {
  int ret=0;

  while(!allZeroParity())	//Do reversals until we hit a hurdle:
    {
    revs.push_back(doRev());
    ++ret;
    }

  if(!allZeroScore())		//If there are hurdles,
    {				//resolve them:
    updateCompInfo();
    ret+=resolveHurdles(revs);	//Get rid of hurdles. (Joel)

    while(!allZeroParity())	//do the remainder of the reversals:
      {
      revs.push_back(doRev());
      ++ret;
      }

    if(!allZeroScore())	     //If there are still hurdles, there is a problem.
      {
      cerr << endl << "++++ERROR: Problem with Handling Hurdles!" << endl;
      cout << endl << "++++ERROR: Problem with Handling Hurdles!" << endl;
      }
    }

  return ret;
  }


/* Update the information about the cycles and components and -gr-.
*/
void Ograph::updateCompInfo()
{
  findGraphArray();
  findCycles();
  comps->classCycles();
  comps->findHurdles();
}

/* Return a lists of all pairs of reality edges that are on different cycles.
*/
void Ograph::getUnsortingInvs(vector<Pair> &ops) const
  {
  Pair p;

  for(int i=1; i < graphsize; i+=2)     //For each reality edge
    for(int j=i+2; j < graphsize; j+=2) //compair it to each r edge after it:
      if(comps->getCycleNum(i) != comps->getCycleNum(j))
        {                               //If the edges are from different cycles
        p = Pair((i+1)/2, j/2);
        ops.push_back(p);               //then add the pair.
        }
  }


/* Return a vector of all pairs of divergent reality edges that represent safe
inversions.  These pairs indicate the inversions that can be done to decrease
the inversion distance.
**This isn't the complete list of sorting inversion if there are hurdles around.
*/
void Ograph::getSafeInvs(vector<Pair> &ops) const
  {
  Pair p;

  for(int i=1; i < graphsize; i+=2)
    {
    if(gr[i].reality.direction == FORWARDS)       //If this edge is FORWARDS
      {
      for(int j=i+2; j < graphsize; j+=2)         //for everyone on this cycle
        if(comps->getCycleNum(j) == comps->getCycleNum(i) &&
           gr[j].reality.direction == BACKWARDS)  //that is backwards
          {
          p = Pair((i+1)/2, j/2);
          if(isSafe(p))
            ops.push_back(p);                     //add the pair.
          }
      }
    else
      {
      for(int j=i+2; j < graphsize; j+=2)
        if(comps->getCycleNum(j) == comps->getCycleNum(i) &&
           gr[j].reality.direction == FORWARDS)
          {
          p = Pair((i+1)/2, j/2);
          if(isSafe(p))
            ops.push_back(p);
          }
      }
    }
  }


/* Return a vector of all pairs of divergent reality edges (with one of the
endpoints being that which is given) that represent oriented
inversions.  These pairs indicate the inversions that can be done to decrease
the inversion distance.  This isn't the complete list of sorting inversion if
there are hurdles around.
*/
void Ograph::getOrientedInvs(vector<Pair> &ops, const int endpoint) const
  {
  Pair p;
  int i=endpoint*2+1;

  assert(i < graphsize);

  if(gr[i].reality.direction == FORWARDS)       //If this edge is FORWARDS
    {
    for(int j=1; j < graphsize; j+=2)         //for everyone on this cycle
      if(comps->getCycleNum(j) == comps->getCycleNum(i) &&
         gr[j].reality.direction == BACKWARDS)  //that is backwards
        {
        if(i<j)
          p = Pair((i+1)/2, j/2);
        else
          p = Pair((j+1)/2, i/2);
        ops.push_back(p);                     //add the pair.
        }
    }
  else                                          //Otherwise it's BACKWARDS...
    {
    for(int j=1; j < graphsize; j+=2)
      if(comps->getCycleNum(j) == comps->getCycleNum(i) &&
         gr[j].reality.direction == FORWARDS)
        {
        if(i<j)
          p = Pair((i+1)/2, j/2);
        else
          p = Pair((j+1)/2, i/2);
        ops.push_back(p);
        }
    }
  }


/* Return a vector of all pairs of non-divergent reality edges from the same
cycle.  These pairs indicate the inversions that can be done that have no
effect on the inversion distance.
*/
void Ograph::getBenignInvs(vector<Pair> &ops) const
  {
  Pair p;

  for(int i=1; i < graphsize; i+=2)
    {
    if(gr[i].reality.direction == FORWARDS)       //If this edge is FORWARDS
      {
      for(int j=i+2; j < graphsize; j+=2)         //for everyone on this cycle
        if(comps->getCycleNum(j) == comps->getCycleNum(i) &&
           gr[j].reality.direction == FORWARDS)   //that is also FORWARDS
          {
          p = Pair((i+1)/2, j/2);
          ops.push_back(p);                       //add the pair.
          }
      }
    else
      {
      for(int j=i+2; j < graphsize; j+=2)
        if(comps->getCycleNum(j) == comps->getCycleNum(i) &&
           gr[j].reality.direction == BACKWARDS)
          {
          p = Pair((i+1)/2, j/2);
          ops.push_back(p);
          }
      }
    }
  }


/*Return a maximum independent set of commuting inversions.
*/
struct IMvertex  //A vertex of the interval model.
  {
  int from;
  int to;     //The 
  boot isleft;   //1 if this is the left vertex of an interval.
  };
void Ograph::getCommutingMIS() const
  {
  //** Current assumption is that we have a single cycle.
  int *forwards = new int[seqsize];        //The forward facing reality edges.
  int fsize=0;
  int *backwards = new int[seqsize];       //The backward facing reality edges.
  int bsize=0;
  for(int i=1; i < seqsize; i++)
    {
    if(gr[2i].direction == FORWARDS)
      forwards[fsize++] = i;
    else
      backwards[bsize++] = i;
    }
  Pair *leftfacing = new Pair[fsize*bsize];
  Pair *rightfacing = new Pair[fsize*bsize];
  int lind=0, rind=0;
  for(int i=0; i < fsize; i++)        //For each forward facing edge,
    {
    int j=0;                          //store intervals to the left
    while(j < bsize && forwards[i] > backwards[j])
      {
      leftfacing[lind++].set(backwards[j++], forwards[i]);
      rightfacing[rind++].set(backwards[j++], forwards[i]);
      }
    while(j < bsize)                  //and then store intervals to the right.
      {
      leftfacing[lind++].set(forwards[i], backwards[j++]);
      rightfacing[rind++].set(forwards[i], backwards[j++]);
      }
    }








/*
  int *cmodel = new int[fsize*bsize*2];    //Allocate the cord model.
  int find=0, bind=0, cind=0;
  while(find < fsize && bind < bsize)
    {                              //Build the cord model as an adjacency list:
    if(forwards[find] < backwards[bind])   //If we're looking at a FORWARD edge,
      {
      for(int i=bind; i >= 0; i--)
        {
        backwards[i];
        }
      }
    }



  int *fmass = new int[seqsize];       //The number of 
  int *bmass = new int[seqsize];       //The backward facing reality edges.
  for(int i=1; i < seqsize; i++)
    {
  while(find < fsize && bind < bsize)
    {
    if(forwards[find] < backwards[bind])
    }

  cmlist *cmodel = new cmlist[fsize*bsize*2]; //Allocate the cord model.
  int find=0, bind=0;
  while(find < fsize && bind < bsize)
    {                              //Build the cord model as an adjacency list:
    if(forwards[find] < backwards[bind])
      {
      
      }
    else
      {
      }
    }
*/
  

  delete [] forwards;
  delete [] backwards;
  }




//%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Inversion %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


/* Do a reversal. Return the interval(inclusive) over which the reversal was
done.
*/
Pair Ograph::doRev()
  {
  int maxScoreVert;		//First to the -matrix- updates:
  Pair currRev;			//Current reversal

  maxScoreVert = getMaxScoreVert();

  adjScoreArray(maxScoreVert);     //Update the score array.
  adjAdjList(maxScoreVert);        //Update the matrix values for neighbors.
  adjParity(maxScoreVert);         //Update the parity values.

  bool* v = matrix[maxScoreVert];
  for(int j = 0; j < matsize; ++j) //Update the value for this vertex.
    v[j] = 0;

                               //Now do the -seq- updates:
//cerr << "\tVERT:" << maxScoreVert << endl;
  int mini = seqinds[maxScoreVert];//Get the interval that the reversal is over.
  int maxi = seqinds[maxScoreVert+1];
//cerr << "\tmini:" << mini << " maxi:" << maxi << endl;
  if(mini > maxi)                  //Make sure it's proper...
    {
    int t = mini;
    mini = maxi;
    maxi = t;
    }

  if(seq[mini] + seq[maxi] > 0)
    {
    rev(mini, maxi - 1);    //Update sequence. Do revAll() to maintain gr.
    currRev = Pair(mini-1, maxi-2);
    }
  else
    {
    rev(mini + 1, maxi);    //Update sequence. Do revAll() to maintain gr.
    currRev = Pair(mini, maxi-1);
    }
#ifdef SORTING_SEQUENCE
  if (globalSortSequence) {
    globalSortSequence->addRev(currRev);
  }
#endif
  return currRev;
  }


/* Reverse -seq- and -seqinds- over the given interval(inclusive).
Take sequence indices.
*/
void Ograph::rev(int min, int max) {
  int t;

  assert(min>0 && max<seqsize-1);   //Make sure we reverse over valid indicies.

  while(min <= max) {
    t = seq[min];
    seq[min] = -seq[max];
    seq[max] = -t;

    seqinds[abs(seq[min])] = min;
    seqinds[abs(seq[max])] = max;
    min++;
    max--;
  }
}

/* Reverse -seq-, -seqinds-, and -gr- over the given interval(inclusive).
   Take a Pair.  Update the component info.
*/
void Ograph::revAll(const Pair& p)
  {
  revAllNoUpdate(p.getLeft(), p.getRight());
  comps->classCycles();
  }

/* Reverse -seq-, -seqinds-, and -gr- over the given interval(inclusive).
   Take sequence indices.  Update the component info.
*/
void Ograph::revAll(int min, int max)
  {
  revAllNoUpdate(min, max); 
  comps->classCycles();
  }

/* Reverse -seq-, -seqinds-, and -gr- over the given interval(inclusive).
   Take sequence indices.  Don't update the component info (for efficiency).
  **Efficiency: Update cycles in a smarter way.
*/
void Ograph::revAllNoUpdate(int min, int max) {
  Node tNode;
  int t;

  //cerr << "\tREV:(" << min << "," << max << ")" << endl;

  rev(min, max);

  min = min*2-1;
  max = max*2;

  while(min < max) {
    t = gr[max].desire.id;
    gr[ gr[min].desire.id ].desire.id = max;
    gr[ t                 ].desire.id = min;

    tNode = gr[min];
    {
      if (min%2==0) {
	tNode.reality.id = max-1;
	gr[max].reality.id = min+1;
      }
      else {
	tNode.reality.id = max+1;
	gr[max].reality.id = min-1;
      }
    }
    gr[min] = gr[max];
    gr[max] = tNode;

    ++min;
    --max;
  }

  findCycles();    // **Efficiency - see above.
}



/* Do the given reversal as though it were done on the inverse graph...
in other words, do an inversion on the identity and make the appropriate
changes to the breakpoint graph.  As usual, the given pair represents the 
1-indexed, inclusive indices.
   Update everything.
   ** We just annihilate everything and rebuild the object based on our
      calculated new sequence.
*/
void Ograph::revAllInv(const Pair& p)
  {
  int len = seqsize-2;
  int tmp[len];
  for(int i=0; i<len; i++)   //Copy the sequence.
    tmp[i] = seq[i+1];

  int map[p.getRight()+1];   //We won't use the first p.getLeft() indices.
  int c=0;
  for(int i=p.getLeft(); i<=p.getRight(); i++) //Build the map.
    map[i] = p.getRight()-c++;


  for(int i=0; i<len; i++)     //Copy the sequence.
    if((abs(tmp[i]) >= p.getLeft()) && (abs(tmp[i]) <= p.getRight()))
      {
      if(tmp[i] > 0)               //If the value here is positive
        tmp[i] = -map[tmp[i]];     // then put the negation of it's map
      else                         // otherwise,
        tmp[i] = map[abs(tmp[i])]; // put it's map.
      }

  del();               //Delete everything.
  //invertSeq(tmp, len); //!!Must be changed if joel's invert(in constructor)is.
  construct(tmp, len); //Construct with the new sequence.
  }




/* Return true if the given pair represents a safe reversal on the
current graph.
  ** This check isn't totally right at the moment because it just compairs
     the number of unoriented component before and after the inversion.
     If the particular component that becomes unoriented seperates two others,
     then this is not a hurdle so is technically "safe".
      `--->  If the hurdles are cleared at the begining instead of when we 
             can't sort any longer then this wouldn't be an issue.
*/
bool Ograph::isSafe(const Pair& p) const
  {
  Ograph tmp = *this;

  int UCcount = tmp.comps->getBadCompCount(); //Get the current UC count.
  tmp.revAll(p);                              //Do the inversion.
  if(UCcount < tmp.comps->getBadCompCount())  //Compair new to old UC count.
    return false; //and return false if we now have more unoriented components.

  return true;  //and return true if we don't have more unoriented components.
  }



/*Do an unoriented reversal. Return the interval(inclusive) over which reversal
was done.
***
 * Precondition: inRev must be all falses with a length of seqsize
 * Postcondition: inRev will be all falses
***/
Pair Ograph::doUnorRev(int left, int right) {
  int node, i, len;
  vector<int> nodes;
  Pair currRev;

  revAllNoUpdate(left, right);    // Do reversal in -seq-, -seqinds-, and -gr-

  for (i=left; i <= right; i++) { // mark graph nodes in reversal into inRev
    node = abs(seq[i]);
    inRev[node] = true;
  }

  len = seqsize-1;
  for (i=0; i < len; ++i)
    if (inRev[i] != inRev[i+1])
      nodes.push_back(i);

  for (i=left; i <= right; i++) { // unmark reversal's graph nodes in inRev
    node = abs(seq[i]);		  // - (reestablishes postcondition)
    inRev[node] = false;
  }

  adjAdjList(nodes);		// Update the matrix values for neighbors.

  adjParity(nodes);		// Update the parity values.

  currRev = Pair(left-1, right-1);
#ifdef SORTING_SEQUENCE
  if (globalSortSequence) {
    globalSortSequence->addUnorRev(currRev);
  }
#endif
  return currRev;
}


//%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Hurdles %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



/* Get rid of all hurdles. Return number of reversals it takes. (Joel)
*/
int Ograph::resolveHurdles(vector<Pair> &revs) {
  int numHurdles = comps->getNumHurdles();
  Hurdle hurdle;
  int left;
  const int nullHurdle = CompInfo::nullHurdle.comp;
  int hurdleToSkip = nullHurdle;
  bool mergesRemain = true;
  bool cutDone = true;

  inRev.clear();
  inRev.resize(seqsize, false); // establish precondition for doUnorRev()

  // If odd number of hurdles
  if (numHurdles % 2 == 1)
    cutDone = doFirstCut(revs, hurdleToSkip);

  // Merge all other hurdles (non-consecutive merges until last two hurdles)
  if (numHurdles > 1) {
    left = getFirstLeft(hurdleToSkip);

    while (mergesRemain && hurdleToSkip != nullHurdle)
      mergesRemain = mergeNextTwo(revs, left, hurdleToSkip);

    while (mergesRemain)
      mergesRemain = mergeNextTwo(revs, left);
  }

  // if there was a non-fortress with odd # of hurdles and a maximal simple
  // hurdle, cut the maximal hurdle now
  if (!cutDone)
    cutMaxHurdle(revs);

  // findOvMatHaveMem();		// reconstruct the overlap matrix
  for(int i = 0; i < matsize; ++i)
    s[i] = compScore(i);	// recalculate the score array from scratch
  
  return (numHurdles+1)/2;	// return the number of reversals done
}


/* Do the first cut/merge of a graph with an odd number of hurdles
 *  If the cut is performed, returns true.
 *  If cut should be done after all other merges, returns false.
 */
bool Ograph::doFirstCut(vector<Pair> &revs, int &hurdleToSkip) {
  Hurdle h;

  // If genome is a fortress
  if (comps->isFortress()) {
    comps->get1stHurdle();	// skip first hurdle
    h = comps->getNextHurdle();	// get second hurdle
    hurdleToSkip = h.comp;

    // merge 2nd hurdle and its neo-hurdle (outermost overarching component)
    revs.push_back(doUnorRev(comps->getNeoLM(), h.min));
  }
  else {
    h = comps->get1stSimHur();	// first simple hurdle
    hurdleToSkip = h.comp;

    // if h is a maximal hurdle
    if (comps->hasMaximal() && h==comps->get1stHurdle())
      return false;		// cut it after all the merges

    // cut the simple hurdle
    // (if left side of hurdle is node i, reverse nodes between i and i+1)
    revs.push_back(doUnorRev(h.min+1, gr[h.min*2].desire.id/2));
  }

  return true;			// cut complete
}


/* Return the left index of the first hurdle-merging reversal
    (if the hurdle-to-skip is passed, set variable to null)
 */
int Ograph::getFirstLeft(int &hurdleToSkip) {
  bool ignoreMaximal = false;
  int nullHurdle = CompInfo::nullHurdle.comp;
  Hurdle h = comps->get1stHurdle();

  if (h.comp==hurdleToSkip) {
    hurdleToSkip = nullHurdle;
    h = comps->getNextHurdle();
    ignoreMaximal = true;
  }

  if (!ignoreMaximal && comps->hasMaximal())
    return comps->getMaxLM();	// left of first rev is max of left side of the
				//  maximal hurdle
  else
    return h.max;		// left of first rev is 1st hurdle's max
}


/* Merge the next two hurdles (skipping the first-cut hurdle if necessary)
 * Returns true if more merges remain to be done
 */
bool Ograph::mergeNextTwo(vector<Pair> &revs, int &left, int &hurdleToSkip) {
  int mid, right;
  int nullHurdle = CompInfo::nullHurdle.comp;

  Hurdle h = comps->getNextHurdle();

  if (h.comp==hurdleToSkip) {
    hurdleToSkip = nullHurdle;
    h = comps->getNextHurdle();
  }

  if (h.isNull) 
    cerr << "++++Ran out of hurdles mid-merge" << endl;
  mid = h.min;			// mid is min of next hurdle

  h = comps->getNextHurdle();
  if (h.comp==hurdleToSkip && hurdleToSkip!=nullHurdle) {
    hurdleToSkip = nullHurdle;
    h = comps->getNextHurdle();
  }

  if (h.isNull) {		// if last merge
    revs.push_back(doUnorRev(left, mid));
    return false;		// no merges remain
  }
  else {			// if not last merge
    right = h.min;		// right is min of next hurdle
    revs.push_back(doUnorRev(left, right));
    left += right-mid;		// mid (in new position) becomes new left
  }

  return true;			// merges remain
}


/* Merge the next two hurdles
 * Returns true if more merges remain to be done
 */
bool Ograph::mergeNextTwo(vector<Pair> &revs, int &left) {
  int mid, right;
  Hurdle h = comps->getNextHurdle();

  if (h.isNull)
    cerr << "++++Ran out of hurdles mid-merge" << endl;
  mid = h.min;			// mid is min of next hurdle

  h = comps->getNextHurdle();
  if (h.isNull) {		// if last merge
    revs.push_back(doUnorRev(left, mid));
    return false;		// no merges remain
  }
  else {			// if not last merge
    right = h.min;		// right is min of next hurdle
    revs.push_back(doUnorRev(left, right));
    left += right-mid;		// mid (in new position) becomes new left
  }

  return true;			// merges remain
}


/* Cut the maximum hurdle in a non-fortress genome with an odd # of hurdles
 */
void Ograph::cutMaxHurdle(vector<Pair> &revs) {
  int left = comps->getMaxRM();
  int mid = gr[left*2].desire.id/2;

  // cut starts on the inner right side of the maximal hurdle and may wrap
  // around the end of the genome
  if (left < mid)
    revs.push_back(doUnorRev(left+1, mid));
  else
    revs.push_back(doUnorRev(mid+1, left));
}





//%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Bergeron %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


/*Determine if a vertex is oriented.
*/
bool Ograph::isOriented(int vertex)
  {
  int parity = 0;
  bool* adjArray;
  adjArray = matrix[vertex];
  for(int i = 0; i < matsize; ++i)
    if(adjArray[i])
      ++parity;

  //fprintf(stderr, "%i %i %i \n", vertex, parity, (parity % 2)); 
  return (parity % 2)!=0;
  }

/*Compute the score for a vertex.
*/
int Ograph::compScore(int vertex)
  {
  int i, score;
  bool* adjArray;

  adjArray = matrix[vertex];
  score = 0;

  for(i = 0; i < matsize; ++i)
    if(adjArray[i])
      {
      if(p[i])           //**isOriented**
        --score;
      else
        ++score;
      }
  
  return score;
  }

/*Get the vertex index with the maximal score.
*/
int Ograph::getMaxScoreVert()
  {
  bool noVal = true;
  int maxVert = 0, i;
  for(i = 0; i < matsize; ++i)
    if(p[i] && (noVal || (s[i] > s[maxVert])))
      {
      maxVert = i;
      noVal = false;
      }

  return maxVert;
  }

/*Adjust the score array given vertex v was changed.
*/
void Ograph::adjScoreArray(int maxScoreVert)
  {
  bool* adj = matrix[maxScoreVert];

  for(int i = 0; i < matsize; ++i)
    if(adj[i])
      ++s[i];

  return;
  }

/*Adjust the adjacency lists when a vertex -vert- is changed.
*/
void Ograph::adjAdjList(int vert)
  {
  int i;
  bool *v;
  bool *w;

  v = matrix[vert];
/*
cerr << "V:   ";
for(i = 0; i < matsize; ++i)
  {
  cerr << v[i] << " ";
  }
*/

  v[vert] = 1;
  for(i = 0; i < matsize; ++i)
    {
//cerr << "\n+++ i:" << i << " - " << v[i] << endl;
    if((i != vert) && v[i])
      {
      w = matrix[i];

      if(!p[i])         //**isOriented**
        adjForW(i, w, v, -1);
      else
        adjForW(i, w, v, 1);
      }
    }

  return;
  }

/*Adjust the adjacency matrix after an unoriented reversal,
 * given the overlap nodes which have had their parity reversed.
 */
void Ograph::adjAdjList(vector<int> nodes) {
  int row, col;
  int size = nodes.size();
  for (int i=size-1; i > 0; --i) {
    row = nodes[i];
    for (int j=i-1; j >= 0; --j) {
      col = nodes[j];
      matrix[col][row] = matrix[row][col] = !matrix[row][col];
    }
  }
}



/*Adjust the adj for the w in the paper.
*/
void Ograph::adjForW(int wInd, bool *w, bool *v, int gainLoss)
  {
  int i;

  //update the score
  for(i = 0; i < matsize; ++i)
    if(w[i])
      s[i] = s[i] + gainLoss;

  w[wInd] = 1;

/*
/--------------------------------------------------------
  fprintf(stderr, "WIND:%i\nv:", wInd);
  for(int j = 0; j < matsize; ++j)
    fprintf(stderr, "%i ", v[j]);
  
  fprintf(stderr, "w:");
  for(int j = 0; j < matsize; ++j)
    fprintf(stderr, "%i ", w[j]);
  //fprintf(stderr, "\n");
/--------------------------------------------------------
*/

  //adjust w with respect to v
//cerr << endl << "W[" << wInd << "]:";
  for(i = 0; i < matsize; ++i)
{
//cerr << w[i] << " ";
    w[i] = (v[i] && !w[i]) || (!v[i] && w[i]);
}

/* 
/--------------------------------------------------------
  fprintf(stderr, "\nw:");
  for(int j = 0; j < matsize; ++j)
    fprintf(stderr, "%i ", w[j]);
  fprintf(stderr, "\n");

  fprintf(stderr, "v:");
  for(int j = 0; j < matsize; ++j)
    fprintf(stderr, "%i ", v[j]);
  fprintf(stderr, "\n");
/--------------------------------------------------------
*/
  
  //do the score thing again
  for(i = 0; i < matsize; ++i)
    if(w[i])
      s[i] = s[i] + gainLoss;
  }


/*Adjust the parity.
*/
void Ograph::adjParity(int v)
  {
  int i;
  bool* vAdj;
  
  vAdj = matrix[v];
  for(i = 0; i < matsize; ++i)
    p[i] = (vAdj[i] && !p[i]) || (!vAdj[i] && p[i]);

  return;
  }



/* Fill -matrix- with Bergeron's overlap matrix for this graph as well as
the score and parity arrays.
*/
void Ograph::findOverlapMat()
  {                                     //Allocate the square matrix:
  matrix = (bool **)malloc(matsize*sizeof(bool*));
  for(int i=0; i < matsize; i++)
    matrix[i] = new bool[matsize];
  s = new int[matsize];                 //Allocate the score and parity arrays:
  p = new bool[matsize];

  findOvMatHaveMem();
  }



/* Fill -matrix- with Bergeron's overlap matrix for this graph as well as
the score and parity arrays. (when memory is already allocated)
*/
void Ograph::findOvMatHaveMem() {
  int i;
  for(i=0; i < matsize; i++)	       //Initialize the matrix:
    for(int j=0; j < matsize; j++)
      matrix[i][j] = 0;
    

  for(i=0; i < graphsize-1; i++)		      //For each
    if(i < gr[i].desire.id)                           //edge
      for(int j = i+1; j < gr[i].desire.id; j++)      //check for overlapping
        if((gr[j].desire.id < i) || (gr[j].desire.id > gr[i].desire.id))
	  {
	    //fprintf(stderr, "\n\n\n--------------------------> %i %i %i\n", matsize, gr[i].label/2, gr[j].label/2);
	    matrix[gr[i].label/2][gr[j].label/2] = true;  //and mark it if so.
	  }

  // fill the score and parity arrays
  for(i = 0; i < matsize; ++i)	       // compScore() uses the parities
    p[i] = isOriented(i);
  for(i = 0; i < matsize; ++i)	       //so do it second.
    s[i] = compScore(i);

  /*
  // print the matrix
  cout << "Ograph matrix:" << endl;
  for(i=0; i < matsize; ++i) {
    for(int j=0; j < matsize; j++)
      cout << matrix[i][j] << " ";
    cout << endl;
  }
  cout << "Parity:" << endl;
  for(i=0; i < matsize; ++i)
    cout << p[i] << " ";
  cout << endl << "Score:" << endl;
  for(i=0; i < matsize; ++i)
    cout << s[i] << " ";
  cout << endl;
  */
}



/*Adjust the parity after an unoriented reversal,
 * given the overlap graph nodes which have had their parity reversed.
 */
void Ograph::adjParity(vector<int> nodes) {
  int node;
  int size = nodes.size();
  for (int i=0; i < size; ++i) {
    node = nodes[i];
    p[node] = !p[node];
  }
}


/*Is the parity 0 for all the verts.
*/
bool Ograph::allZeroParity()
  {
  for(int i = 0; i < matsize; ++i)
    if(p[i])
      return false;

  return true;
  }

/*Is the score 0 for all the verts. (There are hurdles)
*/
bool Ograph::allZeroScore()
  {
  for(int i = 0; i < matsize; ++i)
    if(s[i])
      return false;

  return true;
  }


//%%%%%%%%%%%%%%%%%%%%%%%%%%%%% OUTPUT %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

ostream& operator<<(ostream& output, Ograph& c)
  {
  int i;
  cerr << "| ";
  for(i=0; i < c.seqsize; i++)
    cerr << c.seq[i] << " ";
  cerr << "|";
  for(i=0; i < c.matsize; i++)
    {
    cerr << endl << i << ":";
    for(int j=0; j < c.matsize; j++)
      {
      cerr << c.matrix[i][j] << " ";
      }
    }
  cerr << endl << "-\ns:";
  for(i=0; i < c.matsize; i++)
    cerr << c.s[i] << " ";
  cerr << endl << "p:";
  for(i=0; i < c.matsize; i++)
    cerr << c.p[i] << " ";

  return output;
  }

/* Output the graph is svg format.
*/
void Ograph::printGraphSVG(ostream& filehandle) const
  {
  std::vector<int> g(&seq[1], &seq[seqsize-1]);

  cmb::bp::drawSVG(filehandle, g);
  }


void Ograph::printGraph() const
  {
  int i;

  fprintf(stderr, "--\n");
  fprintf(stderr, "Graph Size = %i\n", graphsize);
  fprintf(stderr, "Seq Size = %i\n", seqsize);
  fprintf(stderr, "Mat Size = %i\n", matsize);
  for(i=0; i < graphsize; i++)
    cerr << gr[i].label << " ";
  cerr << endl;
  for(i=2; i < graphsize; i+=2)
    {
    if(gr[i].label%2)
      {
      cerr << "-";
      cerr << gr[i-1].label/2 << " ";
      }
    else
      cerr << gr[i].label/2 << " ";
    }
  fprintf(stderr, "\n--\n");

/*
  for(i=0; i < graphsize; i++)
    fprintf(stderr, "%d - name:%d real:%d pos?%d des:%d\n", i, gr[i].label, gr[i].reality.id, gr[i].sign, gr[i].desire.id);
*/
  }

/* Print the sequence to stderr.
*/
void Ograph::printSeqStr() const
  {
  for(int i=1; i < seqsize-1; i++)
    cerr << seq[i] << " ";
  }

/* Print the cycles.
*/
void Ograph::printCycles() const
  { comps->printCycles(); }

/* Print the cycles.
*/
void Ograph::printCompInfo() const
  { cerr << *comps << endl; }

/* Print the sequence to stderr with '*' to indicate FORWARDS oriented edges.
*/
void Ograph::printOrientationStr() const
  {
  if(gr[0].reality.direction == FORWARDS)
    cerr << "*";

  for(int i=1; i < seqsize-1; i++)
    {
    cerr << seq[i];
    if(gr[i*2].reality.direction == FORWARDS)
      cerr << "*";
    else
      cerr << " ";
    }
  }

void Ograph::printArray(const int a[], int seqsize) const
  {
  int i;

  for(i=0; i < seqsize; i++)
    fprintf(stderr, "%d ", a[i]);

  fprintf(stderr, "\n");
  }


//-----TEMP DEBUG OUTPUT-----//
void Ograph::printGrInfo() const {
  int i;

  cerr << "----" << endl << "seq:";
  for (i=0; i<seqsize; ++i)
    cerr << seq[i] << " ";
  cerr << endl << "gr.sign:";
  for (i=0; i<seqsize*2-2; ++i)
    cerr << gr[i].sign << " ";

  cerr << endl << "gr.label:";
  for (i=0; i<seqsize*2-2; ++i)
    cerr << gr[i].label << " ";
  cerr << endl << "gr.reality:";
  for (i=0; i<seqsize*2-2; ++i)
    cerr << gr[i].reality.id << " ";
  cerr << endl << "realityref:";
  for (i=0; i<seqsize*2-2; ++i)
    cerr << gr[gr[i].reality.id].label << " ";
  cerr << endl << "gr.reality.direction:";
  for (i=0; i<seqsize*2-2; ++i)
    cerr << gr[i].reality.direction << " ";

  cerr << endl << "gr.reality.deletion:";
  for (i=0; i<seqsize*2-2; ++i)
    cerr << gr[i].reality.deletion << " ";
  cerr << endl << "gr.reality.copies:  ";
  for (i=0; i<seqsize*2-2; ++i)
    cerr << gr[i].reality.copies << " ";

  cerr << endl << "gr.desire: ";
  for (i=0; i<seqsize*2-2; ++i)
    cerr << gr[i].desire.id << " ";

  cerr << endl << endl;

  cerr << *comps << endl;
  cerr << "----" << endl;
}

/* Return a string from the given input.
*/
template < class T >
string Ograph::ToString(const T &arg) const
  {
  ostringstream out;
  out << arg;

  return(out.str());
  }


