/* Krister Swenson                                                 Winter 2007-8

A simple graph class that knows it's Max Clique and Max IS
and can decompose itself by the join decomposition.
*/

#include "Graph.h"

using namespace std;
using namespace __gnu_cxx;

Graph::Graph(bool u)
  {
  undirected = u; largest = LARGEST;
  CM_ENUM.resize(6);
  setinit=false;
  }
Graph::Graph(const Graph &g)
  { copy(g); }


/*Innitialize a graph with a chord model (double occurance word).
*/
Graph::Graph(const vector<int> &cm)
  {
  vector<string> blank;
  init(cm, blank);
  }


/*Innitialize a graph with a chord model (double occurance word).
 * Use -chordTOinv- to set the name of the inversions.
*/
Graph::Graph(const vector<int> &cm, const vector<string> chordTOinv)
  { init(cm, chordTOinv); }

/*Innitialize a graph with a chord model (double occurance word).
 * Use -chordTOinv- to set the name of the inversions.
*/
void Graph::init(const vector<int> &cm, const vector<string> chordTOinv)
  {
  undirected = true;
  largest = LARGEST;
  CM_ENUM.resize(6);
  setinit=false;
            //Add the vertices:
  if(!chordTOinv.empty())             //If the chord->inversion map is given
    for(vector<int>::const_iterator i=cm.begin(); i < cm.end(); i++)
      addVertex(*i, chordTOinv[*i]);  //then add the name to the vertex.
  else                                //Otherwise, label it with integer only.
    for(vector<int>::const_iterator i=cm.begin(); i < cm.end(); i++)
      addVertex(*i);

            //Add the edges:
  for(unsigned int i=0; i < cm.size(); i++)
    {                        //For each chord
    hash_map< int, int > beenseen;
    unsigned int j=i+1;
    for(j=i+1; j < cm.size() && cm[i] != cm[j]; j++)
      {
      if(beenseen.find(cm[j]) != beenseen.end())
        beenseen[cm[j]]++;
      else
        beenseen[cm[j]] = 1;
      }
    if(j < cm.size())        //If we're looking at the first endpoint.
      for(hash_map< int, int >::iterator k = beenseen.begin();
          k != beenseen.end(); k++)
        if(k->second == 1)
          addEdge(cm[i], k->first);
    }
  }

void Graph::operator=(const Graph &g)
  { vset.clear(); copy(g); }
Graph::~Graph()
  {}

/*Copy the graph.
 ** currently we lose track of the names!
*/
void Graph::copy(const Graph &g)
  {
  undirected = false;        //Make undirected so that each edge is copied once.
  for(vertmap::const_iterator i = g.vset.begin(); i != g.vset.end(); i++)
    {
    const Vertex *v = &i->second;                  //A vertex v in g.
    vector<int> ns;                                //For each neighbor of v,
    for(vpmap::const_iterator j=v->nBeg(); j != v->nEnd(); j++)
      ns.push_back(j->second->getLabel());         //record it as a neighbor.
    addVertex(v->getLabel(),ns,false);             //Add vertex and neighbors.
    }
  undirected = g.undirected; //Set type of graph after copy.
  largest = g.largest;
  CM_ENUM = g.CM_ENUM;
  maxsets = g.maxsets;
  setinit = g.setinit;
  }


/*Add a single edge between -l1- and -l2-.  Add the vertices if they don't 
already exist.
*/
void Graph::addEdge(int l1, int l2)
  {
  Vertex* x = addVertex(l1);
  Vertex* y = addVertex(l2);
  addEdgeExisting(x,y);
  }

/*Add a single edge between -v- and -w-.  Add the vertices if they don't 
already exist.
*/
void Graph::addEdge(const Vertex *v, const Vertex *w)
  {
  Vertex* x = addVertex(v->getLabel(), v->getName());
  Vertex* y = addVertex(w->getLabel(), w->getName());
  addEdgeExisting(x,y);
  }


/*Add an edge between existing vertices.
*/
void Graph::addEdgeExisting(Vertex *v, Vertex *w)
  {
  v->addNeighbor(w);
  if(undirected)
    w->addNeighbor(v);
  }

/*Remove an edge if it exists.
*/
void Graph::remEdge(int l1, int l2)
  {
  if(vset.find(l1) != vset.end() && vset.find(l2) != vset.end())
    remEdge(&vset[l1], &vset[l2]);
  }

/*Remove an edge if it exists.
*/
void Graph::remEdge(Vertex *v, Vertex *w)
  {
  v->remNeighbor(w);
  if(undirected)
    w->remNeighbor(v);
  }



/*Add a single vertex labelled -l- with no edges.
*/
Vertex* Graph::addVertex(int l)
  {
  if(vset.find(l) == vset.end())
    vset[l] = Vertex(l);                //Create l if need be.
  return &vset[l];
  }

/*Add a single vertex named -n- and labelled -l-, with no edges.
*/
Vertex* Graph::addVertex(int l, const std::string &name)
  {
  if(vset.find(l) == vset.end())
    vset[l] = Vertex(l, name);          //Create l if need be.
  return &vset[l];
  }



/*Add a vertex labelled -l- adjacent to those in -adjlist-.  Any vertex that is
in -adjlist- but does not exist will be created. Ignore self edges.
If -checkdup- is false then don't check for duplicate vertex labels.
*/
void Graph::addVertex(int l, const vector<int> &adjlist, bool checkdup)
  {
  Vertex *v;
  if(checkdup && vset.find(l) != vset.end())
    v = &vset[l];                       //Use the existing vertex.
  else                                  //Otherwise create a new vertex:
    {
    vset[l] = Vertex(l);
    v = &vset[l];
    }

  for(unsigned int i=0; i<adjlist.size(); i++)
    {
    int nind = adjlist[i];
    if(nind != l)
      {
      if(vset.find(nind) == vset.end()) //Create the current one if need be.
        vset[nind] = Vertex(nind);

      v->addNeighbor(&vset[nind]);      //Make l adjacent to the current one.
      if(undirected)                    //For an undirected graph,
        vset[nind].addNeighbor(v);      //do the other direction as well.
      }
    }
  }

/*Add a vertex labelled -l- adjacent to those in -adjset-.  Any vertex that is
in -adjset- but does not exist will be created. Ignore self edges.
If -checkdup- is true then don't check for duplicate vertex labels.
*/
void Graph::addVertex(int l, const hash_set<int> &adjset, bool checkdup)
  {
  vector<int> nb;             //The neighbors of l will be in here.
  for(labelset::iterator i = adjset.begin(); i != adjset.end(); i++)
    nb.push_back(*i);         //Mark vertices in adjset.
  addVertex(l, nb, checkdup);
  }

/*Remove the vertex with label l.  Return the neighbors of -l-.
*/
hash_set<int> Graph::remVertex(int l)
  {
  hash_set<int> neighbors;

  vertmap::iterator v = vset.find(l);
  if(v != vset.end())
    {
    vpmap::const_iterator i = v->second.nBeg();
    while(i != v->second.nEnd())    //Erase all edges with this vertex,
      {
      vpmap::const_iterator tmp = i++;
      neighbors.insert(tmp->second->getLabel());
      remEdge(&(v->second), tmp->second);
      }
    vset.erase(v);                  //and then erase this vertex.
    }

  return neighbors;
  }

/*Remove all the neighbors of the given vertex.
*/
void Graph::remNeighbors(int l)
  {
  Vertex* v = &vset[l];
  vpmap::const_iterator j = v->nBeg();
  while(j != v->nEnd())
    {
    vpmap::const_iterator tmp = j++;
    remVertex(tmp->second->getLabel());
    }
  }


/* Build a bipartite graph from vertices labelled between l1-l2 to those
labelled r1-r2.
*/
void Graph::buildBipartite(int l1, int l2, int r1, int r2)
  {
  vector<int> l,r;
  for(int i=l1; i<=l2 ; i++)
    l.push_back(i);
  for(int i=r1; i<=r2 ; i++)
    r.push_back(i);
  for(vector<int>::iterator i = l.begin(); i != l.end(); i++)
    addVertex(*i, r);
  for(vector<int>::iterator i = r.begin(); i != r.end(); i++)
    addVertex(*i, l);
  }

/* Build a clique among the given vertices.
*/
void Graph::buildClique(int v1, int v2)
  {
  for(int i=v1; i<v2; i++)
    for(int j=i+1; j<=v2; j++)
      addEdge(i,j);
  }

/*Return the intersection of -g- and this graph.
*/
Graph Graph::getIntersection(const Graph &g) const
  {
  Graph retg;                                   //For each vertex in this graph,
  for(vertmap::const_iterator i = vset.begin(); i != vset.end(); i++)
    {
    if(g.hasVertex(i->second.getLabel()))
      retg.addVertex(i->second.getLabel(), i->second.getName());
    for(vpmap::const_iterator j = i->second.nBeg(); j != i->second.nEnd(); j++)
      if(g.hasEdge(i->second.getLabel(), j->second->getLabel()))  //if g has it,
        retg.addEdge(&i->second, j->second);                      //then add it.
    }
  return retg;
  }

/*Return the union of -g- and this graph.
*/
Graph Graph::getUnion(const Graph &g) const
  {
  Graph retg;                                   //For each edge in this graph,
  for(vertmap::const_iterator i = vset.begin(); i != vset.end(); i++)
    {
    retg.addVertex(i->second.getLabel(), i->second.getName());
    for(vpmap::const_iterator j = i->second.nBeg(); j != i->second.nEnd(); j++)
      retg.addEdge(&i->second, j->second);
    }
                                                //Do the same for g:
  for(vertmap::const_iterator i = g.vset.begin(); i != g.vset.end(); i++)
    {
    retg.addVertex(i->second.getLabel(), i->second.getName());
    for(vpmap::const_iterator j = i->second.nBeg(); j != i->second.nEnd(); j++)
      retg.addEdge(&i->second, j->second);
    }

  return retg;
  }




/*____________________________________________________________________________*/



/* Return true if the given edge exists in this graph.
*/
bool Graph::hasEdge(Vertex *v, Vertex *w) const
  { return v->hasNeighbor(w); }
bool Graph::hasEdge(int l, int r) const
  { return hasEdge(ipair(l, r)); }
bool Graph::hasEdge(const ipair e) const
  {
  vertmap::const_iterator v;
  if((v = vset.find(e.first)) != vset.end())
    return v->second.hasNeighbor(e.second);

  return false;
  }

/*Return true if the given vertex exists in this graph.
*/
bool Graph::hasVertex(int v) const
  { return vset.find(v) != vset.end(); }


/*Return a sublist of -l- that contains only the vertex labels that are in this
graph.
*/
list<int> Graph::getVertexSubset(const list<int> &l) const
  {
  list<int> retval;
  for(list<int>::const_iterator i = l.begin(); i != l.end(); i++)
    if(hasVertex(*i))
      retval.push_back(*i);
  return retval;
  }



/*Get the maximum independent set of vertices using circle graphs is possible.
  ** not finished!
*/
vector<int> Graph::getMIS()
  {
  cerr << "NOT FINISHED!" << endl;
  int _NOT_FINISHED_YET;
  assert(vset.size() > 3);
  vector<int> theMIS;

  vector<Graph> d;
  decompose(d);                          //Find the decomposition.
  for(vector<Graph>::iterator g = d.begin(); g != d.end(); g++)
    {
    list<int> CM;
    list<int> MIS;
    if(g->getCG(CM))                     //Get the chord model for the graph g.
      getMISCG(CM, MIS);
    else
      MIS = getMISEX();
    cout << *g;
    cerr << MIS << endl;
    }

  return theMIS;
  }

/*Return true if this graph is a circle graph.
*/
bool Graph::isCG()
  {
  vector<int> cm = getCM();
  return !cm.empty();
  }

/* Return a chord model (double occurance word) for the given graph.
An empty model is returned if non exist.
*/
vector<int> Graph::getCM()
  {
  vector<int> theCM;
  if(vset.size() == 0)
    return theCM;
  else if(vset.size() == 1)
    {
    theCM.push_back(vset.begin()->second.getLabel());
    return theCM;
    }

  vector<Graph> d;
  decompose(d);                          //Find the decomposition.
  vector< list<int> > CMs;               //The vector of chord models.
  for(vector<Graph>::iterator g = d.begin(); g != d.end(); g++)
    {
    list<int> CM;
    if(!g->getCG(CM))                    //Get the chord model for the graph g.
      return theCM;

    CMs.push_back(CM);
    }

  theCM = mergeCMs(CMs);                 //Merge the chord models:
  int _TEMPORARY_SANITY_CHECK;
  assert(checkCM(theCM));    //TERMPORARY (sanity check):

  return theCM;
  }




/* Return true if there are no vertices in this graph.
*/
bool Graph::empty() const
  { return vset.empty(); }

/* Return the number of vertices in the subgraph.
*/
int Graph::size() const
  { return vset.size(); }


/* Return true if one vertex is reachable from the other.
*/
bool Graph::isReachable(Vertex &v, const Vertex &target)
  {
  v.mark();
  for(vpmap::const_iterator j=v.nBeg(); j != v.nEnd(); j++)
    {
    if(j->second == &target)           //If we found the target vertex.
      return true;                     //return true,
    else if(!j->second->getMark())     //otherwise recurse to unmarked vertices.
      isReachable(*(j->second), target);
    }
  return false;
  }

/*Return a copy of the connected components.
*/
vector<Graph> Graph::getComponents()
  {
  vector<Graph> retval;

  for(vertmap::iterator i = vset.begin(); i != vset.end(); i++)
    i->second.mark();                       //Mark all vertices in vset.

  int count=0;
  for(vertmap::iterator i = vset.begin(); i != vset.end(); i++)
    {
    Vertex *v = &i->second;                 //A vertex v in g.
    if(v->getMark())                        //If v is marked
      {
      retval.push_back(Graph());            //then add a new component,
      retval[count].undirected = undirected;
      retval[count].largest = largest;
      buildConSubgraph(retval[count++], *v);//and form the component.
      }
    }
  return retval;
  }

/*Get the subgraph comprised of the components with the given vertices.
*/
Graph Graph::getCompsWith(const list<int> &set)
  {
  Graph retval;

  for(list<int>::const_iterator i = set.begin(); i != set.end(); i++)
    markComponent(vset[*i]);

  for(list<int>::const_iterator i = set.begin(); i != set.end(); i++)
    buildConSubgraph(retval, vset[*i]);

  return retval;
  }


/*Return true if the given graph has the same edge set as this one.
*/
bool Graph::operator==(const Graph &g)
  {
  for(vertmap::const_iterator i = g.vset.begin(); i != g.vset.end(); i++)
    {
    if(vset.find(i->second.getLabel()) == vset.end())
      return false;
    else if(vset[i->second.getLabel()] != i->second)
      return false;
    }
  return true;
  }




/*_____________________________________________________________________________
_/ \_/ \_/ \_/ \_/ \_/ \_/ \_/ \_/ \_/ \_/ \_/ \_/ \_/ \_/ \_/ \_/ \_/ \_/ \_/ \
PRIVATE MEMBERS:                                                              */





/*Recursively decompose the graph with the join decomposition.
  Return true if the graph could be decomposed at least once.
  ** this seperates the components.
     `-> for efficiency we could skip this for known single components
         like when doing primality testing.
*/
bool Graph::decompose(vector<Graph> &d)
  {
  bool ret = false; 

  vector<Graph> comps = getComponents();
  for(vector<Graph>::iterator i = comps.begin(); i != comps.end(); i++)
    {
    ipset totest;                         //The edges to test.
    vertmap::iterator j = i->vset.begin();
    Vertex *v = &(j->second);
    sEdges(*v, totest);                   //Fill the edge Q.
    clean(*v);

    ret = decompose(d, *i, totest) || ret;//Get decomposition on this component.
    }

  return ret; 
  }
/* ** assume that the graph is a single component.
*/
bool Graph::decompose(vector<Graph> &d, Graph &g, ipset &edgeS)
  {
  vector<Graph> s = g.split(edgeS, largest); //Get a split on g.
  if(s[0].empty())                           //If we didn't find one
    {
    d.push_back(g);                          //add the prime graph to d,
    return false;                            //and notify the caller.
    }

  decompose(d, s[0], edgeS);                 //Try to decompose the first part.
  decompose(d, s[1], edgeS);                 //Try to decompose the second part.

  return true;                               //We found a split so return true.
  }



/*Put each edge of the connected component connected to -v- into -s-.
  ** each edge goes in exactly once (only one of two directions)
*/
void Graph::sEdges(Vertex &v, ipset &s)
  {
  v.mark();
  for(vpmap::const_iterator i=v.nBeg(); i != v.nEnd(); i++)
    if(!i->second->getMark())        //For each marked neighbor of v,
      {                              //add the edge
      s.insert(ipair(v.getLabel(), i->second->getLabel()));
      sEdges(*(i->second), s);       //and explore the other end's neighbors.
      }
  }

/*Update the set of edges to be checked so that the edges in given
bipartite vertex parition will connect only to -l-.
*/
void Graph::updateEdgeSet(ipset &checkset, const labelset b1, const labelset b2,
                          int l)
  {
  ipset::iterator last = checkset.end();
  ipset::iterator e = checkset.begin();
  while(e != last)
    {                                 //For an edge that crossed the split
    if(((b1.find(e->first) != b1.end()) && (b2.find(e->second) != b2.end()))
       ||
       ((b1.find(e->second) != b1.end()) && (b2.find(e->first) != b2.end())))
      {
      ipset::iterator old = e++;      //then add the edge for both sides.
      checkset.insert(ipair(old->first, l));
      checkset.insert(ipair(old->second, l));
      checkset.erase(old);
      }
    else
      e++;
    }
  }


/*Return a split of the this graph by the join decomposition.  In the new
 graphs the marker vertex will be labelled -l-.
  (Find a bipartition so that all edges between for a complete bipartite graph)
*/
vector<Graph> Graph::split(ipset &testset, int &l)
  {
  ipset::iterator e = testset.begin();
  while(e != testset.end())
    {
    if(hasEdge(*e))        //For each queued edge (x,y) in this graph
      {                    //test for a split xz|y and yz|x:
      for(vertmap::iterator i = vset.begin(); i != vset.end(); i++)
        {
        Vertex *z = &(i->second);
        labelset s1, s2;   //The split in vertices.
        labelset b1, b2;   //The bipartite subgraphs of the split.

        if(z->getLabel() != e->first && z->getLabel() != e->second)
          {
        if(z->getLabel() != e->first && z->getLabel() != e->second)
          if(testSplit(s1,s2, b1,b2, vset[e->first], vset[z->getLabel()],
                                     vset[e->second]) ||
             testSplit(s1,s2, b1,b2, vset[e->second], vset[z->getLabel()],
                                     vset[e->first]) )
            {        //If there is a split with at least 2 on each side then:
            vector<Graph> retpair;
            retpair.push_back(getSubgraph(s1));//Get the subgraphs,
            retpair.push_back(getSubgraph(s2));
            updateEdgeSet(testset, b1, b2, l); //Update the edges to be checked.

            retpair[0].addVertex(l, b1);       //and add the marker vertex.
            retpair[1].addVertex(l++, b2);

            return retpair;
            }
          }
        }

      ipset::iterator old = e++;
      testset.erase(old);  //Erase the edge from further consideration.
      }
    else
      e++;
    }

  vector<Graph> r;
  r.push_back(Graph());
  r.push_back(Graph());
  return r;                //Otherwise, return a blank pair.
  }
  


/*Test for a split with -x- and -z- on one side and -y- on the other.
 If there is a split then fill the sets -V1- and -V2- with vertex labels and
 -V1B- and -V2B- with those that participate in the bipartite graph.
*/
bool Graph::testSplit(labelset &V1, labelset &V2, labelset &V1B, labelset &V2B,
                      Vertex &x, Vertex &z, Vertex &y)
  {
  queue<int> QV1;   //queue for those in V1.
  V1.clear();       //Those in set 1.
  V1B.clear();      //Those in set 1 that take part in the bipartite graph.
  V2B.clear();      //Those in set 2 that may take part in the bipartite graph.
  V2.clear();       //Those that may be in set 2.

  vertexset VS; 

  V1B.insert(x.getLabel());  //We guess that x is in V1B.
  QV1.push(z.getLabel());    //We know z is in V1 U V1B.
  for(vertmap::iterator i = vset.begin(); i != vset.end(); i++)
    {                        //Neighbors of x are in V2B, other in V2:
    Vertex *v = &i->second;             //The current vertex.
    if(v->getLabel() != x.getLabel() && v->getLabel() != z.getLabel())
      {                                 //If we're not looking at x or z
      if(v->hasNeighbor(x.getLabel()))  //and v is a neighbor of x
        V2B.insert(v->getLabel());      //then make it a possible V2B member.
      else                              //Otherwise,
        V2.insert(v->getLabel());       //make it a possible V2 member.
      }
    }

  while(!QV1.empty())
    {
    int v = QV1.front();
    QV1.pop();
    if(y.hasNeighbor(v))      //If v is a neighbor of y then all of its
      {                       //neighbors thought to be in V2 aren't.
      V1B.insert(v);          //We know v is in V1B.
      for(vertmap::iterator i = vset.begin(); i != vset.end(); i++)
        {
        Vertex w = i->second;                  //The current vertex.
        if(w.hasNeighbor(v))                   //If w is a neighbor of v,
          {
          if(V2.erase(w.getLabel()) > 0)       //and it's in V2
            QV1.push(w.getLabel());            //then remove it and queue it.
          }
        else                                   //If w is not a neighbor of v,
          {
          if(V2B.erase(w.getLabel()) > 0)      //and it's in V2B
            QV1.push(w.getLabel());            //then remove it and queue it.
	  }
        }
      }
    else                      //If v is not a neighbor of y then all of its
      {                       //neighbors must be in V1:
      V1.insert(v);           //We know v is in V1.
      for(vpmap::const_iterator i=vset[v].nBeg(); i != vset[v].nEnd(); i++)
        {                                          //For each neighbor,
        if(V2.erase(i->second->getLabel()) > 0)    //if it's in V2
          QV1.push(i->second->getLabel());         //remove it and queue it.
        if(V2B.erase(i->second->getLabel()) > 0)
          QV1.push(i->second->getLabel());
        }
      }
    }

  if((V1.size()+V1B.size() < 2) || (V2.size()+V2B.size() < 2))
    return false;

  V1.insert(V1B.begin(), V1B.end());
  V2.insert(V2B.begin(), V2B.end());
  return true;
  }

/* Complement the graph in the neighborhood of -v-.
*/
void Graph::complement(const Vertex &v)
  {
  vpmap::const_iterator i = v.nBeg();
  while(i != v.nEnd())
    {
    vpmap::const_iterator j = i;
    j++;
    while(j != v.nEnd())
      {
      if(hasEdge(i->second, j->second))
        remEdge(i->second, j->second);
      else
        addEdgeExisting(i->second, j->second);
      j++;
      }
    i++;
    }
  }



/* Return a subgraph of this graph on only those vertices in -set-.
*/
Graph Graph::getSubgraph(const labelset &set)
  {
  Graph retG;
  for(labelset::iterator i = set.begin(); i != set.end(); i++)
    vset[*i].mark();                    //Mark vertices in vset.

  for(labelset::iterator i = set.begin(); i != set.end(); i++)
    if(vset[*i].getMark())              //If the vertex is still marked 
      buildConSubgraph(retG, vset[*i]); //get the marked connected subgraph.

  return retG;
  }

/* Recursively build the connected subraph from marked vertices.  Unmark them.
*/
void Graph::buildConSubgraph(Graph &g, Vertex &v)
  {
  vector<int> uns;    //Unmarked neighbors.
                      //Add the vertex and it's marked neighbors:
  v.unmark();                                   //Unmark it as visited.
                                                //For each neighbor,
  for(vpmap::const_iterator i=v.nBeg(); i != v.nEnd(); i++)
    if(i->second->getMark())                    //if it's marked
      uns.push_back(i->second->getLabel());     //record it as a neighbor.
  g.addVertex(v.getLabel(),uns);                //Add the vertex and neighbors.
                                                //For each neighbor,
  for(vpmap::const_iterator i=v.nBeg(); i != v.nEnd(); i++)
    if(i->second->getMark())                    //if it's marked
      buildConSubgraph(g, *(i->second));        //then visit it.
  }


/* Mark all vertices reachable from -v-.
*/
void Graph::markComponent(Vertex &v)
  {
  v.mark();
  for(vpmap::const_iterator i=v.nBeg(); i != v.nEnd(); i++)
    if(!i->second->getMark())
      markComponent(*(i->second));
  }



/* Unmark v's connected component induced by considering only marked vertices.
*/
void Graph::clean(Vertex &v)
  {
  v.unmark();
  for(vpmap::const_iterator i=v.nBeg(); i != v.nEnd(); i++)
    if(i->second->getMark())         //For each maked neighbor,
      clean(*(i->second));           //unmark it and it's neighbors.
  }

/* Unmark every vertex in the graph.
*/
void Graph::clean()
  {
  for(vertmap::iterator i = vset.begin(); i != vset.end(); i++)
    i->second.unmark();
  }


/* Get a chord model (double occurance word) for the given graph.
Return false if there isn't one.
*/
bool Graph::getCG(list<int> &retval)
  {
  if(size() == 0)
    return false;
  else if(size() == 1)
    {
    retval.push_back(vset.begin()->second.getLabel());
    retval.push_back(vset.begin()->second.getLabel());
    return true;
    }
  else if(size() > 5)
    {
              //Try the graph G-v first:
    vertmap::iterator i = vset.begin();
    while(i != vset.end())
      {
      int v = i->second.getLabel();    //The vertex we're looking at.
      hash_set<int> vns;               //Keeps the neighbors of v.

      i++;                             //move the iterator before removing v,
      vns = remVertex(v);              //remove v while keeping the neighbors,
      Graph g = *this;
      addVertex(v, vns);               //and restore the vertex,

      vector<Graph> garbage;
      if(!g.decompose(garbage))        //then see if it is prime.
        {
        if(!g.getCG(retval))           //If it is prime, get the chord model.
          return false;
        for(list<int>::iterator j=retval.begin(); j != retval.end(); j++)
          {                            //Now look to place v appropriately:
          list<int>::iterator tmp = j;
          for(list<int>::iterator k = ++tmp; k != retval.end(); k++)
            {                          //Starting with a length 1 interval,
            addToCM(v, retval, j, k);  //add the vertex label to retval,
            if(checkCM(retval))        //and check if it matches this graph.
              return true;
            remFromCM(retval, j, k);   //add the vertex label to retval,
            }
          }

        cerr << "it's prime but has no representation!" << endl;
        cout << *this;
        return false;
        }
      }

              //Try the graph G*v-v second:
    i = vset.begin();
    while(i != vset.end())
      {
      int v = i->second.getLabel();    //The vertex we're looking at.
      hash_set<int> vns;               //Keeps the neighbors of v.

      complement(i->second);           //Complement the neighborhood,
      i++;                             //move the iterator before removing v,
      vns = remVertex(v);              //remove v while keeping the neighbors,
      Graph g = *this;
      addVertex(v, vns);               //restore the vertex
      complement(vset[v]);             //and recomplement,

      vector<Graph> garbage;
      if(!g.decompose(garbage))        //then see if it is prime.
        {
        if(!g.getCG(retval))           //If it is prime, get the chord model.
          return false;
        for(list<int>::iterator j=retval.begin(); j != retval.end(); j++)
          {                            //Now look to place v appropriately:
          list<int>::iterator tmp = j;
          for(list<int>::iterator k = ++tmp; k != retval.end(); k++)
            {                          //Starting with a length 1 interval,
            list<int>::iterator newl = k; newl--;
            reverseCM(retval, j, k);     //reverse the interval from j to k,
            addToCM(v, retval, newl, k); //add the vertex label to retval,
            if(checkCM(retval))          //and check if it matches this graph.
              return true;
            remFromCM(retval, newl, k);  //add the vertex label to retval,
            reverseCM(retval, newl, k);  //reverse the interval from j to k,
            }
          }

        cerr << "it's prime but has no representation!" << endl;
        cout << *this;
        return false;
        }
      }

              //Try the graph G*v*w*v-v third:
    cerr << "NEITHER G-v or G*v-v worked, IMPLEMENT G*v*w*v-v!" << endl;
    exit(1);
    }
  else
    {
    if(CM_ENUM[size()].empty())     //If the possible CMs don't exist
      {
      vector<int> b = innitBucket(size());
      fillCMenum(b);                //then enumerate them now.
      }

                                    //Remap the vertex labels:
    vector<int> remap;     //Maps values 1...size() to vertex labels.
    for(vertmap::const_iterator i = vset.begin(); i != vset.end(); i++)
      remap.push_back(i->second.getLabel());
    vector< list<int> > cm_enum = CM_ENUM[size()];
    for(vector< list<int> >::iterator i = cm_enum.begin();
        i != cm_enum.end(); i++)
      for(list<int>::iterator j = i->begin(); j != i->end(); j++)
         *j = remap[*j];
       
    for(vector< list<int> >::iterator i = cm_enum.begin();
        i != cm_enum.end(); i++)
      {
      if(checkCM(*i))                      //Now check if one works.
        {
        retval.splice(retval.begin(), *i); //If it does then put it into retval.
        return true;
        }
      }
    cerr << "ERROR: couldn't find a chord model for size < 5 graph.\n";
    return false;
    }

  cerr << "ERROR: got to the end of getCG!" << endl;
  exit(1);                                 //We should never get here!
  }

/* Check whether the given chord model is consistent with -g-.
*/
bool Graph::checkCM(const list<int> &cm)
  {
  if(cm.empty() && vset.empty())
    return true;
  else if(cm.empty())
    return false;

  for(list<int>::const_iterator i = cm.begin(); i != cm.end(); i++)
    {
    hash_map<int, int> cc;           //Times a chord is seen in the interval.
    list<int>::const_iterator tmp = i;
    list<int>::const_iterator l = ++tmp;
    while(l != cm.end() && *l != *i)
      cc[*(l++)] = 0;
    if(l != cm.end())                //If *i is the left endpoint:
      {
      for(list<int>::const_iterator j = tmp; *j != *i; j++)
        cc[*j]++;                    //Count # of times we see a chord.
                                     //(if once then we overlap)
  
             //Make sure all edges in the model are consistent with the vertex:
      for(hash_map<int, int>::iterator j = cc.begin(); j != cc.end(); j++)
        if((j->second == 1) && !hasEdge(*i, j->first))
          return false;        //If the intervals overlap but edge isn't there.

             //Make sure that incident edges are consistent with the model:
      Vertex *vert = &vset[*i];                   //For each edge check that
      for(vpmap::const_iterator k=vert->nBeg(); k != vert->nEnd(); k++)
        if((cc.find(k->second->getLabel()) == cc.end()) ||
           (cc[k->second->getLabel()] != 1))      //we see it exactly once.
          return false;
      }
    }

  return true;
  }
bool Graph::checkCM(const vector<int> &cm)
  {
  list<int> lcm;
  for(vector<int>::const_iterator i = cm.begin(); i != cm.end(); i++)
    lcm.push_back(*i);

  return checkCM(lcm);
  }


/*Reverse the subword given by the interval [l,r).
*/
void Graph::reverseCM(list<int> &cm, list<int>::iterator l,
                                     list<int>::iterator r) const
  {
  list<int> holder;
  holder.splice(holder.begin(), cm, l, r);   //Put the interval in -holder-.
  holder.reverse();                          //Reverse the interval.
  cm.splice(r, holder);                      //Put the interval back into -cm-.
  }

/*Add the given "vertex" to the word before l and before r.
*/
void Graph::addToCM(int v, list<int> &cm, list<int>::iterator l,
                                          list<int>::iterator r) const
  {
  cm.insert(l, v);
  cm.insert(r, v);
  }


/*Remove the characters from the word before l and before r.
*/
void Graph::remFromCM(list<int> &cm, list<int>::iterator l,
                                     list<int>::iterator r) const
  {
  cm.erase(--l);
  cm.erase(--r);
  }


/*Merge the give chord models through the markers.
*/
vector<int> Graph::mergeCMs(vector< list<int> > &cmlist)
  {             //Creat the maps for referencing the markers.
  hash_map< int, vector< list<int>::iterator > > markermap;//Map marker to iter.
  hash_map< int, vector< list<int>* > > cmmap;             //Maps marker to cm.
  for(vector< list<int> >::iterator i = cmlist.begin(); i != cmlist.end(); i++)
    for(list<int>::iterator j = i->begin(); j != i->end(); j++)
      if(*j >= LARGEST)              //Find all markers so that we get them with
        {
        markermap[*j].push_back(j);  //markermap[x][0,1] and markermap[x][2,3],
        cmmap[*j].push_back(&(*i));  //and cmmap[x][0,1] and cmmap[x][2,3].
        }
                //Merge chord models through the markers:
  while(largest > LARGEST)
    {
    largest--;
                   //Merge the second word into the first word:
    list<int> *first, *second;
    list<int>::iterator firstleft, firstright, secondleft, secondright; 
    first = cmmap[largest][0];
    second = cmmap[largest][2];
    firstleft = markermap[largest][0];
    firstright = markermap[largest][1];
    secondleft = markermap[largest][2];
    secondright = markermap[largest][3];

                          //Record those markers to be moved to the first word:
    vector< list<int>::iterator > marksmoved;
    for(list<int>::iterator i = second->begin(); i != second->end(); i++)
      if(*i >= LARGEST && *i < largest)
        marksmoved.push_back(i);

                          //Replace the first marker in the first word with
                          //those between the two in the second word:
    list<int>::iterator left = secondleft; left++;
    first->splice(firstleft, *second, left, secondright);
    first->erase(firstleft);
                          //Replace the second marker in the first word with
                          //those on the outside in the second word:
    left = secondright; left++;
    first->splice(firstright, *second, left, second->end());
    first->splice(firstright, *second, second->begin(),secondleft);
    first->erase(firstright);
    second->clear();
                          //Update cmmap pointer to the second go to the first.
    for(vector< list<int>::iterator >::iterator i = marksmoved.begin();
        i != marksmoved.end(); i++)
      {
      if(markermap[**i][0] == *i)
        cmmap[**i][0] = first;
      if(markermap[**i][1] == *i)
        cmmap[**i][1] = first;
      if(markermap[**i][2] == *i)
        cmmap[**i][2] = first;
      if(markermap[**i][3] == *i)
        cmmap[**i][3] = first;
      }
    }

                //Any nonempy words are seperate components so concatinate:
  vector<int> fullCM;       //The return value.
  for(vector< list<int> >::iterator i = cmlist.begin(); i != cmlist.end(); i++)
    if(!i->empty())
      for(list<int>::iterator j = i->begin(); j != i->end(); j++)
        fullCM.push_back(*j);

  return fullCM;
  }

/*Return the second highest marker in the given double occurance word.
  ** we assume the highest in -doc- is -largest-.
*/
int Graph::nextHighestMark(const list<int> &doc) const
  {
  int secondhighest = 0;
  for(list<int>::const_iterator i = doc.begin(); i != doc.end(); i++)
    if(*i >= LARGEST && *i < largest && *i > secondhighest)
      secondhighest = *i;
  return secondhighest;
  }


/* Check whether the given vertex is consistent with -cc-.
bool Graph::checkVertex(int v, const vector<int> &cc) const
  {      //Make sure all edges in the model are consistent with the graph:
  for(unsigned int j=0; j<cc.size(); j++)
    {
    if((cc[j] == 1) && !g.hasEdge(v, j)) //If the interval endpoints overlap
      return false;                      //but the edge is not there, die.
    else if(g.hasEdge(v, j))             //Otherwise, they don't overlap, so
      return false;                      //die if the edge is there.
    }

         //Now make sure that all graph edges are consistent with the model:
  Vertex *vert = vset[v];
  for(int j=0; j > v.size(); j++)
    if(cc[neighbor[j]->getLabel()] != 1)
      return false;

  return true;
  }
*/






/*Exhaustively search for a maximum clique on the graph.  -goal- is the maximum
size clique to search for.
*/
list<int> Graph::getMCEX(int goal) const
  {
  hash_map<int,int> map;             //Map labels to [1,n].
  vector<int> invmap;                //Map [1,n] to labels.
  invmap.push_back(0);
  int newlabel=0;
  for(vertmap::const_iterator i = vset.begin(); i != vset.end(); i++)
    {
    map[i->second.getLabel()] = newlabel++;
    invmap.push_back(i->second.getLabel());
    }

  vector< vector <int> > mat;
  vector<int> labels;
  vector<int> degrees;

  mat.resize(vset.size());           //Innitialize the matrix:
  for(unsigned int i=0; i < vset.size(); i++)
    mat[i].resize(vset.size());

  for(vertmap::const_iterator i = vset.begin(); i != vset.end(); i++)
    {                                                      //For each vertex.
    labels.push_back(i->second.getLabel());
    degrees.push_back(i->second.getDegree());
    for(vpmap::const_iterator j = i->second.nBeg(); j != i->second.nEnd(); j++)
      mat[map[i->second.getLabel()]][map[j->second->getLabel()]] = 1; //add it.
    }
  
  list<int> retval = maxclique(mat, labels, degrees, goal);
  for(list<int>::iterator i = retval.begin(); i != retval.end(); i++)
    *i = invmap[*i];                 //Map the labels back.

  return retval;
  }

/*Exhaustively search for a maximum independent set on the graph.  -goal- is the
maximum size independent set to look for.
*/
list<int> Graph::getMISEX(int goal) const
  {
  hash_map<int,int> map;             //Map labels to [1,n].
  vector<int> invmap;                //Map [1,n] to labels.
  invmap.push_back(0);
  int newlabel=0;
  for(vertmap::const_iterator i = vset.begin(); i != vset.end(); i++)
    {
    map[i->second.getLabel()] = newlabel++;
    invmap.push_back(i->second.getLabel());
    }

  vector< vector <int> > mat;
  vector<int> labels;
  vector<int> degrees;

  mat.resize(vset.size());                     //Innitialize the matrix:
  for(unsigned int i=0; i < vset.size(); i++)
    mat[i].resize(vset.size());

  for(vertmap::const_iterator i = vset.begin(); i != vset.end(); i++)
    {                                          //For each vertex.
    labels.push_back(i->second.getLabel());
    degrees.push_back(i->second.getDegree());
    for(vpmap::const_iterator j = i->second.nBeg(); j != i->second.nEnd(); j++)
      mat[map[i->second.getLabel()]][map[j->second->getLabel()]] = 1; //add it.
    }
  for(unsigned int i=0; i<vset.size(); i++)    //Invert the adjacency matrix:
    for(unsigned int j=0; j<vset.size(); j++)
      {
      if(mat[i][j])
        mat[i][j] = 0;
      else
        mat[i][j] = 1;
      }
    
  
  list<int> retval = maxclique(mat, labels, degrees, goal);
  for(list<int>::iterator i = retval.begin(); i != retval.end(); i++)
    *i = invmap[*i];                 //Map the labels back.

  return retval;
  }


/* Get a list of all the independent sets in the graph.
 ** use the igraph library.
*/
vector< list<int> > Graph::getAllIS()
  {
  vector< list<int> > retval;

  hash_map<int,int> map;             //Map labels to [1,n].
  vector<int> invmap;                //Map [1,n] to labels.
  int newlabel=0;                    //Make the labels start at 0:
            //(labels may not be consecutive because of unsafe vertex removal)
  for(vertmap::const_iterator i = vset.begin(); i != vset.end(); i++)
    {
    map[i->second.getLabel()] = newlabel++;
    invmap.push_back(i->second.getLabel());
    }
  
  igraph_t graph;
  igraph_empty(&graph, size(), false);        //Create the graph.
  igraph_vector_t edges;
  igraph_vector_init(&edges, size()*size()*2);

  int index=0;    //The vector index where we will put the edge (two ints).
  for(vertmap::const_iterator i = vset.begin(); i != vset.end(); i++)
    for(vpmap::const_iterator j = i->second.nBeg(); j != i->second.nEnd();j++)
      {           //Add each edge as the next two elements of edges vector.
      VECTOR(edges)[index++] = map[i->second.getLabel()];
      VECTOR(edges)[index++] = map[j->second->getLabel()];
      }
  igraph_add_edges(&graph, &edges, 0);        //Now add the edges to the graph.

  igraph_vector_ptr_t result;                 //Get the independent sets:
  igraph_vector_ptr_init(&result, 0);
  igraph_independent_vertex_sets(&graph, &result, 0,0);

  int numis = igraph_vector_ptr_size(&result);
  for(int i=0; i < numis; i++)
    {
    igraph_vector_t* v;                     //Get the vector
    v = (igraph_vector_t*)igraph_vector_ptr_e(&result,i);

    int issize = igraph_vector_size(v);     //and then store it:
    retval.push_back(list<int>());
    for(int i=0; i < issize; i++)
      retval.back().push_back(invmap[VECTOR(*v)[i]]);

    igraph_vector_destroy(v);
    free(v);
    }

  igraph_vector_ptr_destroy(&result);
  igraph_vector_destroy(&edges);
  igraph_destroy(&graph);
  return retval;
  }


/* Get a list of all the maximal independent sets in the graph.
*/
vector< list<int> > Graph::getAllMalIS()
  {
  hash_map<int,int> map;             //Map labels to [1,n].
  vector<int> invmap;                //Map [1,n] to labels.
  int newlabel=0;                    //Make the labels start at 0:
            //(labels may not be consecutive because of unsafe vertex removal)
  for(vertmap::const_iterator i = vset.begin(); i != vset.end(); i++)
    {
    map[i->second.getLabel()] = newlabel++;
    invmap.push_back(i->second.getLabel());
    }
  
  vector< vector <int> > mat; //The matrix representing the graph.
  mat.resize(vset.size());                     //Innitialize the matrix:
  for(unsigned int i=0; i < vset.size(); i++)
    mat[i].resize(vset.size());
  for(vertmap::const_iterator i = vset.begin(); i != vset.end(); i++)
    {                                          //Add each vertex:
    for(vpmap::const_iterator j = i->second.nBeg(); j != i->second.nEnd();j++)
      mat[map[i->second.getLabel()]][map[j->second->getLabel()]] = 1;
    }

  
  vector< list<int> > retval = getAllMaximal(vset.size(), mat);
  for(vector< list<int> >::iterator j=retval.begin(); j!=retval.end(); j++)
    for(list<int>::iterator i = j->begin(); i != j->end(); i++)
      *i = invmap[*i];        //Map the labels back.

  return retval;
  }

/* Get the next maximal independent set in the graph.
*/
/*
list<int> Graph::getNextMalIS()
  {
  if(!setinit)           //If the list is uninitialized, then initialize it:
    {
    setinit = true;

    hash_map<int,int> map;             //Map labels to [1,n].
    vector<int> invmap;                //Map [1,n] to labels.
    int newlabel=0;                    //Make the labels start at 0:
    for(vertmap::const_iterator i = vset.begin(); i != vset.end(); i++)
      {
      map[i->second.getLabel()] = newlabel++;
      invmap.push_back(i->second.getLabel());
      }
  
    vector< vector <int> > mat;
  
    mat.resize(vset.size());                     //Innitialize the matrix:
    for(unsigned int i=0; i < vset.size(); i++)
      mat[i].resize(vset.size());
  
    for(vertmap::const_iterator i = vset.begin(); i != vset.end(); i++)
      {                                          //For each vertex.
      for(vpmap::const_iterator j = i->second.nBeg(); j != i->second.nEnd();j++)
        mat[map[i->second.getLabel()]][map[j->second->getLabel()]] = 1;//add it.
      }
    
    maxsets = getAllMaximal(vset.size(), mat);
    for(vector< list<int> >::iterator j=maxsets.begin(); j!=maxsets.end(); j++)
      for(list<int>::iterator i = j->begin(); i != j->end(); i++)
        *i = invmap[*i];                 //Map the labels back.
    }

  list<int> retval;
  if(!maxsets.empty())
    {
    retval = maxsets.back();
    maxsets.pop_back();
    }

  return retval;
  }
*/


/*Get the next maximal independent set in the graph with the given
vertices in -whitelist- and without the given vertices in -blacklist-.
*/
/*
list<int> Graph::getMalISWith(const list<int> whitelist,
                              const list<int> blacklist)
  {
  //We need to do this over all possible orderings of adding vertices.
  //Would johnson's algorithm help?
  //We can remove the nodes and their neighbors.  Then run recursively.
cerr << "whitelist:" << whitelist << endl;
cerr << "blacklist:" << blacklist << endl;

  Graph g = *this;
  g.resetMalISinfo();
  for(list<int>::const_iterator i=whitelist.begin(); i != whitelist.end(); i++)
    {
    g.remNeighbors(*i); //Remove the neighbors of those in the white list.
    //remVertex(*i)      //Remove the vertices in the white list.
    }
  for(list<int>::const_iterator i=blacklist.begin(); i != blacklist.end(); i++)
    g.remVertex(*i);    //Remove the vertices in the black list.


                        //Now get the new maximal ISs:
  list<int> set = g.getNextMalIS();
  list<int> old;
  while(!set.empty())
    {
    old = set;
    cerr << "\tnew set:" << set << endl;
    set = g.getNextMalIS();
    }

  return old;
  }
*/


/*Reset datastructures related to maximal independent sets.
*/
void Graph::resetMalISinfo()
  { setinit = false; maxsets.clear(); }

/*Return true if the given subset of vertices constitute an independent set.
*/
bool Graph::isIS(const list<int> &set) const
  {
  for(list<int>::const_iterator v = set.begin(); v != set.end(); v++)
    {
    //Vertex *vp = &(vset[*v]);
    vertmap::const_iterator vp = vset.find(*v);
    list<int>::const_iterator w = v;
    while(++w != set.end())
      //if(vp->hasNeighbor(vset[*w]))
      if(vp->second.hasNeighbor(&(vset.find(*w)->second)))
        return false;
    }

  return true;
  }



/*Find one IS from another IS by replacing in -is- a vertex at or after the one
pointed to by -p- with one not in -is-.
Return true and have -p- point to the replaced value if a valid substitution
was made.
*/
bool Graph::ISreplace(list<int> &is, list<int>::iterator &p) const
  {
  hash_set<int> ish;
  for(list<int>::const_iterator i = is.begin(); i != is.end(); i++)
    ish.insert(*i);

  while(p != is.end())
    {                           //For each member of the independent set
    int isv = *p;
    list<int>::iterator tmp = p; tmp++;
    is.erase(p);                //remove it and test if there is a vertex
    for(vertmap::const_iterator v = vset.begin(); v != vset.end(); v++)
      {                         //independent of the other vertices:
      if((ish.find(v->second.getLabel()) == ish.end()) &&
         !v->second.hasNeighbor(is))             //If there is no neighbor then
        {
        p = is.insert(tmp, v->second.getLabel());//make the replacement.
        return true;
        }
      }
    is.insert(tmp, isv);
    p = tmp;
    }

  return false;
  }








/* Recursively fill in the CM_ENUM array (of the appropriate size) with 
all possible chord models.
  ** we only need to check the combinations with a fixed first element.
*/
void Graph::fillCMenum(vector<int> &bucket)
  {
  list<int> v;
  v.push_back(0);       //Fix the first element.
  (*bucket.begin())--;
  fillCMenumh(v, bucket);
  (*bucket.begin())++;
  }
void Graph::fillCMenumh(list<int> &v, vector<int> &bucket)
  {
  if(v.size() == bucket.size()*2)
    CM_ENUM[bucket.size()].push_back(v);

  int location = 0;
  for(vector<int>::iterator i = bucket.begin(); i != bucket.end(); i++)
    {
    if(*i)                   //If there is an element in this bucket.
      {
      v.push_back(location); //then assign it to the current position
      (*i)--;
      fillCMenumh(v, bucket);//and recurse.
      (*i)++;
      v.pop_back();
      }
    location++;
    }
  }

/* Innitialize the buckets to each have value 2.
*/
vector<int> Graph::innitBucket(int size)
  {
  vector <int> retval;

  retval.resize(size);
  for(vector<int>::iterator i=retval.begin(); i != retval.end(); i++)
    *i = 2;

  return retval;
  }

/* Return the number of chord models of the given size (# chords).
*/
int Graph::getEnumSize(int size)
  {
  switch(size)
    {
    case 2: return 6;
    case 3: return 90;
    case 4: return 2520;
    case 5: return 113400;
    default: cerr << "ENUM: broken"; exit(1);
    }
  }


/* Print the decomposition given in -d-.
*/
void Graph::printDecomposition(ostream& output, const vector<Graph> &d) const
  {
  if(undirected)
    output << "graph G" << endl;
  else
    output << "digraph G" << endl;

  output << "  {\n  "
         << "size = \"10,7\";"
         << "orientation=landscape;"
         << "center=false;\n  "
         << "concentrate=true;"
         << "splines=true;"  //Turn this false for much faster graph rendering.
         << "overlap=false;"
         << "ranksep=1.0;"
         << "bgcolor=transparent;"
         << endl;
  
  for(vector<Graph>::const_iterator j = d.begin(); j != d.end(); j++)
    {
    for(vertmap::const_iterator i = j->vset.begin(); i != j->vset.end(); i++)
      {
      output << "  \"" << i->second.getLabel() << "\"" << endl;
      for(vpmap::const_iterator j=i->second.nBeg(); j != i->second.nEnd(); j++)
        output << "  \"" << i->second.getLabel() << "\" -- \""
               << j->second->getLabel() << "\"" << endl;
      }
    output << endl;
    }
  output << "  }\n";
  }



/* Traverse the graph and print the edges.
*/
ostream& operator<<(ostream& output, const Graph& g)
  {
  if(g.undirected)
    output << "graph G" << endl;
  else
    output << "digraph G" << endl;

  output << "  {\n  "
         << "size = \"10,7\";"
         << "orientation=landscape;"
         << "center=false;\n  "
         << "concentrate=true;"
         << "splines=true;"  //Make this false for much faster graph rendering.
         << "overlap=false;"
         << "ranksep=1.0;"
         << "bgcolor=transparent;"
         << endl;
  for(vertmap::const_iterator i = g.vset.begin(); i != g.vset.end(); i++)
    {
    output << "  \"" << i->second.getName() << "\"" << endl;
    for(vpmap::const_iterator j = i->second.nBeg(); j != i->second.nEnd(); j++)
      output << "  \"" << i->second.getName() << "\" -- \""
             << j->second->getName() << "\"" << endl;
    }
  output << "  }\n";

  return output;
  }

/*Print every vertex and a new line.
*/
void Graph::printVertex(ostream& output) const
  {
  for(vertmap::const_iterator i = vset.begin(); i != vset.end(); i++)
    cerr << i->second << endl;
  }

/* Print a labelset.
*/
ostream& operator<<(ostream& output, const labelset &vs)
  {
  for(labelset::iterator i = vs.begin(); i != vs.end(); i++)
    output << *i << " ";
  return output;
  }
/* Print a ipset.
*/
ostream& operator<<(ostream& output, const ipset &ps)
  {
  for(ipset::iterator i = ps.begin(); i != ps.end(); i++)
    output << *i << " ";
  return output;
  }
